/*      
 THE POLAR METHOD FOR COMPUTING A NORMALLY DISTRIBUTED RANDOM
 NUMBER GIVEN A UNIFORMLY DISTRIBUTED RANDOM NUMBER GENERATOR.
 ALGORITHM P, PAGES 103-104
 KNUTH, D. E. (1969)
 THE ART OF COMPUTER PROGRAMMING - VOL 2
 SEMINUMERICAL ALGORITHMS
*/
  
#include <cmath>

//#include "DIGRand.h"
#include <DIGRand/DIGRand.h>
//#include "DIGGauss.h"
#include <DIGRand/DIGGauss.h>

    
     
double Gauss(double mean, double std)
{
  static bool    sw = false;
  static double  s, v1, v2;

  if(!sw) {
    sw = true;
    do {
      v1 = (double) 2.0 * Rand() - (double) 1.0;
      v2 = (double) 2.0 * Rand() - (double) 1.0;
      s = v1 * v1 + v2 * v2;
    }
    while (s >= 1.0);

    s = (double) sqrt((double) -2.0 * (double) log(s) / s);

    return v1 * s * std + mean;
  }

  sw = false;

  return v2 * s * std + mean;
}     
