/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/bwray.cpp $
 $LastChangedRevision: 85 $
 $Date: 2014-07-02 16:07:08 -0400 (Wed, 02 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 this subroutine is the WRAY for blobs

 */

#include <cstdio>
#include <cmath>

#include "consts.h"
#include "blkdta.h"
#include "blob.h"
#include "posit.h"
#include "anglst.h"

#include "uiod.h"

#include "raylen.h"

void Blob_class::bwray(INTEGER np, INTEGER nr, INTEGER* list, REAL* weight,
		INTEGER* numb, REAL* snorm)
{
	INTEGER a, b;
	INTEGER ac, bc, bc1, ac1;
	INTEGER af, al;
	INTEGER bf, bl;
	INTEGER laf, lal;
	INTEGER lbf, lbl;
	INTEGER h, m;
	INTEGER lmf, lml;
	INTEGER h1, m1;
	INTEGER mcov;
	REAL n;
	INTEGER nint;
	INTEGER sign;
	INTEGER M2, H2;

	REAL ax, ay, axid, ayid;
	REAL mx, my;
	REAL theta, sinth, costh;
	REAL prfx, prlx, prfy, prly;

	REAL x, y, yup, ylo;
	REAL ddeltain, ddeltaout;
	REAL dc, dc1;
	REAL ydelta; //xdelta not used

	REAL slope, islope;
	REAL coef, icoef;
	REAL ord, cte, ctes;
	REAL fcosang;
	REAL samp_support;
	REAL multf;

	*numb = 0;

	*snorm = 0;

	posit(np, nr, &ax, &ay, &mx, &my);

	if (0 >= raylen(SOT_rect, ax, ay, mx, my, 0, 0, GeoPar.nelem * GeoPar.pixsiz / 2.0, GeoPar.nelem * GeoPar.pixsiz / 2.0, 1, 0))
	{
		return; // bug 111, if ray(np, nr) miss all pixels in the image, return numb=0, by wei, 6/2005
	}

	multf = Blob.support / Blob.delta;

	samp_support = (REAL) samp / Blob.support;

	laf = (INTEGER) (-0.5 * ((int) (Blob.M / 2) + (int) (Blob.H / 2)));
	lal =
			(INTEGER) (0.5
					* ((int) ((Blob.M - 1) / 2) + (int) ((Blob.H - 1) / 2)));

	lbf = -1 * (INTEGER) (Blob.H / 2);
	lbl = (INTEGER) ((Blob.H - 1) / 2);

	lmf = -1 * (INTEGER) (Blob.M / 2);
	lml = (INTEGER) ((Blob.M - 1) / 2);

	prfx = -0.25 * (Blob.M - 1.0);
	prlx = (-1) * prfx;

	prfy = -0.25 * Consts.sqrt3 * (Blob.H - 1.0);
	prly = (-1) * prfy;

	H2 = (INTEGER) (Blob.H / 2);
	M2 = (INTEGER) (Blob.M / 2);
	axid = ax / Blob.delta;
	ayid = ay / Blob.delta;
	sinth = my;
	costh = mx;

	if (mx == 0)
		theta = Consts.pi * 0.5;
	else
		theta = (REAL) atan2(my, mx);

	if (theta < 0)
		theta += Consts.pi * 2; //use atan2 instead of acos for more acuracy,the range of atan2 is [-pi,pi], by wei 1/2005

	//added 7/19, think of theta as always being betw 0 and pi
	if (theta > Consts.pi)
	{
		theta = theta - Consts.pi;
		sinth = -sinth;
		costh = -costh; //never used anyway
		my = -my;
		mx = -mx;
	}

	if (fabs(my) > Consts.zero)
	{  // non-horizontal lines

		islope = mx / my;
		ord = (axid - islope * ayid);
		coef = 0.5 * (Consts.sqrt3 * islope - 1);

		if (theta < Consts.pi / 6 || theta > 2.0 * Consts.pi / 3.0)
		{  // type 1

			cte = multf / my; // for the "side rays"
			slope = my / mx;
			icoef = 1.0 / coef;
			fcosang = (REAL) fabs(cos(theta + Consts.pi / 6.0));

			mcov = (INTEGER) ceil(multf / fcosang);
			ddeltain = Blob.delta * fcosang;
			ddeltaout = Blob.delta * sinth;

			if (theta < Consts.pi / 6)
			{
				sign = 1;
			}
			else
			{
				sign = -1;
			}

			ctes = cte * sign;

			x = prfx;
			yup = (x - ord + ctes) * slope;
			ylo = (x - ord - ctes) * slope;

			if (sign == 1)
			{ // + slope

				if (yup < prfy)
				{
					yup = prfy;
					x = ord - ctes + islope * yup;
				}

				ac = (INTEGER) floor(x - yup * Consts.isqrt3);
			}
			else
			{ //sign == -1

				ac = (INTEGER) floor(x - yup * Consts.isqrt3);
				if (ylo > prly)
				{
					ylo = prly;
					x = ord + ctes + islope * ylo;
					ac = (INTEGER) floor(x - ylo * Consts.isqrt3);
				}
			}

			af = MAX0(laf, ac);

			x = prlx;
			yup = (x - ord + ctes) * slope;
			ylo = (x - ord - ctes) * slope;

			if (sign == 1)
			{ // + slope

				if (ylo > prly)
				{
					ylo = prly;
					x = ord + ctes + islope * yup;
				}
				ac = (INTEGER) ceil(x - ylo * Consts.isqrt3);
			}
			else
			{ //sign == -1

				ac = (INTEGER) ceil(x - ylo * Consts.isqrt3);
				if (yup < prfy)
				{
					yup = prfy;
					x = ord - ctes + islope * yup;
					ac = (INTEGER) ceil(x - yup * Consts.isqrt3);
				}
			}
			al = MIN0(lal, ac);
			a = af;
			bc = (INTEGER) (floor(icoef * (a - ord)));

			bf = MAX0(bc - mcov, lbf);
			bl = MIN0(bc + mcov, lbl);

			x = a + 0.5 * bc;
			y = 0.5 * Consts.sqrt3 * bc;
			dc = Blob.delta * (REAL) fabs(-my * (axid - x) + mx * (ayid - y));

			bc1 = bc + 1;
			dc1 = ddeltain - dc;

			// initial values
			b = bc;
			n = dc;
			m = 2 * a + b;
			h = b;
			m1 = m + M2;
			h1 = h + H2;

			//starting the loops--------------------------------

			while (a <= al)
			{
				if (bf <= bl)
				{
					while (b >= bf)
					{

						if (m >= lmf && m <= lml && b <= lbl)
						{

							if (n < Blob.support)
							{

								list[*numb] = h1 * Blob.M + m1;
								nint = (INTEGER) round(samp_support * n); // changed "rint" to "round". Lajos, Jan 25, 2005

								weight[*numb] = Blob.ltable[nint];

								*snorm += weight[*numb] * weight[*numb];
								*numb = *numb + 1;
							}
						}
						b--;
						m--;
						m1--;
						h1--;
						n += ddeltain;
					}

					b = bc1;
					n = dc1;
					m = 2 * a + b;
					h = b;
					m1 = m + M2;
					h1 = h + H2;

					while (b <= bl)
					{

						if (m >= lmf && m <= lml && b >= lbf)
						{
							if (n < Blob.support)
							{

								list[*numb] = h1 * Blob.M + m1;
								nint = (INTEGER) round(samp_support * n); // changed "rint" to "round". Lajos, Jan 25, 2005

								weight[*numb] = Blob.ltable[nint];

								*snorm += weight[*numb] * weight[*numb];
								*numb = *numb + 1;
							}
						}
						b++;
						m++;
						// h++;
						m1++;
						h1++;
						n += ddeltain;
					}
				} //if bf<= bl

				dc = dc + (REAL) sign * ddeltaout;

				dc1 = ddeltain - dc;

				if (dc > ddeltain || dc < 0.0)
				{
					bc = bc + sign;
					dc = (REAL) sign * (fabs(dc) - ddeltain);
					bc1 = bc + 1;
					dc1 = ddeltain - dc;
					bf = MAX0(bc - mcov, lbf);
					bl = MIN0(bc + mcov, lbl);
				}

				a++;
				b = bc;
				n = dc;
				m = 2 * a + b;
				h = b;
				m1 = m + M2;
				h1 = h + H2;
			} //while a
		}   //type 1

		else
		{   // type 2
			fcosang = (REAL) fabs(cos(theta + Consts.pi / 6.0));
			mcov = (INTEGER) ceil(multf / sinth);
			// constants independent of the first grid point
			bf = lbf;
			bl = lbl;
			ddeltain = Blob.delta * sinth;
			ddeltaout = Blob.delta * fcosang;

			if (theta < Consts.pi / 3)
			{
				sign = 1;
			}
			else
			{
				sign = -1;
			}

			//----------------------------------------------
			// parameters of the first grid point

			b = bf;

			ac = (INTEGER) (floor(coef * b + ord));

			af = MAX0(ac - mcov, laf);
			al = MIN0(ac + mcov, lal);

			x = ac + 0.5 * b;
			y = 0.5 * Consts.sqrt3 * b;
			dc = Blob.delta * (REAL) fabs(-my * (x - axid) + mx * (y - ayid));

			ac1 = ac + 1;
			dc1 = ddeltain - dc;

			//init values
			a = ac;
			n = dc;
			m = 2 * a + b;
			h = b;
			m1 = m + M2;
			h1 = h + H2;

			// starting the loop .....................

			while (b <= bl)
			{
				if (af <= al)
				{
					while (a >= af)
					{

						if (m >= lmf && m <= lml)
						{

							if (n < Blob.support)
							{
								list[*numb] = h1 * Blob.M + m1;
								nint = (INTEGER) round(samp_support * n); // changed "rint" to "round". Lajos, Jan 25, 2005

								weight[*numb] = Blob.ltable[nint];

								*snorm += weight[*numb] * weight[*numb];
								*numb = *numb + 1;
							}
						}
						a--;
						m -= 2;
						m1 -= 2;
						n += ddeltain;
					}

					a = ac1;
					n = dc1;
					m = 2 * a + b;
					h = b;
					m1 = m + M2;
					h1 = h + H2;

					while (a <= al)
					{
						if (m >= lmf && m <= lml)
						{
							if (n < Blob.support)
							{

								list[*numb] = h1 * Blob.M + m1;

								nint = (INTEGER) round(samp_support * n); // changed "rint" to "round". Lajos, Jan 25, 2005

								weight[*numb] = Blob.ltable[nint];

								*snorm += weight[*numb] * weight[*numb];
								*numb = *numb + 1;
							}
						}
						a++;
						m += 2;
						m1 += 2;
						n += ddeltain;
					}
				} //if af <= al

				dc = dc + (REAL) sign * ddeltaout;

				dc1 = ddeltain - dc;

				if (dc > ddeltain || dc < 0.0)
				{
					ac = ac + sign;
					dc = (REAL) sign * (fabs(dc) - ddeltain);

					ac1 = ac + 1;
					dc1 = ddeltain - dc;

					af = MAX0(ac - mcov, laf);
					al = MIN0(ac + mcov, lal);
				}

				b++;
				a = ac;
				n = dc;
				m = 2 * a + b;
				h = b;
				m1 = m + M2;
				h1 = h + H2;
			} //while b
		}  // type 2
	}         // if non- horizontal line.

	else
	{  //if horizontal line (my == 0)

		fcosang = cos(Consts.pi / 6.0);

		mcov = (INTEGER) ceil(multf / fcosang);

		// constants independent of the first grid point
		ddeltain = Blob.delta * fcosang;

		//parameters of the first grid point
		//find optimum af

		x = prfx;
		yup = ayid + multf;
		ylo = yup - 2 * multf;

		ac = (INTEGER) floor(x - yup * Consts.isqrt3);

		af = MAX0(laf, ac);

		//find optimum al

		x = prlx;
		ac = (INTEGER) ceil(x - ylo * Consts.isqrt3);
		al = MIN0(lal, ac);

		//done with af and al

		a = af;
		bc = (INTEGER) (floor(2 * ayid * Consts.isqrt3));
		bf = MAX0(bc - mcov, lbf);
		bl = MIN0(bc + mcov, lbl);

		y = 0.5 * Consts.sqrt3 * bc;
		ydelta = y * Blob.delta;
		dc = (REAL) fabs(ydelta - ay);
		bc1 = bc + 1;
		dc1 = ddeltain - dc;

		// init values
		b = bc;
		n = dc;
		//n = nc;
		m = 2 * a + b;
		h = b;
		m1 = m + M2;
		h1 = h + H2;

		while (a <= al)
		{
			if (bf <= bl)
			{
				while (b >= bf)
				{

					if (m >= lmf && m <= lml && b <= lbl)
					{
						if (n < Blob.support)
						{

							list[*numb] = h1 * Blob.M + m1;
							nint = (INTEGER) round(samp_support * n); // changed "rint" to "round". Lajos, Jan 25, 2005

							weight[*numb] = Blob.ltable[nint];
							*snorm += weight[*numb] * weight[*numb];
							*numb = *numb + 1;
						}
					}
					b--;
					m--;

					m1--;
					h1--;
					n += ddeltain;
				}

				b = bc1;
				n = dc1;
				m = 2 * a + b;
				h = b;
				m1 = m + M2;
				h1 = h + H2;

				while (b <= bl)
				{
					if (m >= lmf && m <= lml && b >= lbf)
					{
						if (n < Blob.support)
						{

							list[*numb] = h1 * Blob.M + m1;
							nint = (INTEGER) round(samp_support * n); // changed "rint" to "round". Lajos, Jan 25, 2005

							weight[*numb] = Blob.ltable[nint];

							*snorm += weight[*numb] * weight[*numb];
							*numb = *numb + 1;
						}
					}
					b++;
					m++;

					m1++;
					h1++;
					n += ddeltain;
				}
			} // if bf <= bl

			a++;
			b = bc;
			n = dc;
			//n = nc;
			m = 2 * a + b;
			h = b;
			m1 = m + M2;
			h1 = h + H2;

		} //while a
	}  // fabs (my)==0

	if (trace > 6)
	{
		fprintf(output,
				"\n          bwray    np = %5i  nr = %5i  numb = %5i  snorm = %10.4f",
				np, nr, *numb, *snorm);
	}

	if ((*numb > 0) && (trace > 9))
	{
		for (int i = 0; i < *numb; i++)
		{

			fprintf(output, "\n                  %5i     %10.4f", list[i],
					weight[i]);  //bug 139
		}
	}
	return;
}
