/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/contur.cpp $
 $LastChangedRevision: 85 $
 $Date: 2014-07-02 16:07:08 -0400 (Wed, 02 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 A PROCEDURE TO CONTOUR A RECONSTRUCTION INTO A BLACK-WHITE PICTURE
 */

#include <cstdio>

#include "blkdta.h"
#include "geom.h"
#include "math.h" //added. hstau
#include "consts.h"
#include "uiod.h"

#include "contur.h"

void contur(REAL* a, INTEGER m, INTEGER n, REAL thresh, REAL w1, REAL w2, REAL w3)
{
	INTEGER i, last = m * n;
	REAL t = thresh;

	if (w3 >= Consts.zero)
	{
		REAL min, max, amin, amax, ai, at;

		if ((GeoPar.aveden <= w1) || (GeoPar.aveden >= w2))
		{
			fprintf(output,
					"\n **** invalid parameters to contour - (aveden <= w1 or w2 <= aveden)");
			fprintf(output, "\n **** contour failed to continue");
			return;
		};

		// find min and max
		min = a[0];
		max = a[0];
		for (i = 1; i < last; i++)
		{
			ai = a[i];
			if (ai < min)
				min = ai;
			if (ai > max)
				max = ai;
		};

		if ((max - min) < Consts.zero)
		{
			fprintf(output, "\n **** picture is a constant");
			fprintf(output, "\n **** contour failed to continue");
			return;
		};

		// perform a bisection search
		min = min - 0.00001;
		amin = w2;
		amax = w1;
		while ((amin != GeoPar.aveden) && (amax != GeoPar.aveden) && (max - min > 0.00001))
		{
			t = (min + max) / 2.0;
			at = 0;
			for (i = 0; i < last; i++)
			{
				if (a[i] <= t)
					at += w1;
				else
					at += w2;
			};
			at /= last;
			if (at < GeoPar.aveden)
			{
				max = t;
				amax = at;
			}
			else
			{
				min = t;
				amin = at;
			}
		};

		if (amin - GeoPar.aveden < GeoPar.aveden - amax)
			t = min;
		else
			t = max;

		fprintf(output, "\n         contour threshold = %10.4f", t);
	};

	for (i = 0; i < last; i++)
	{
		if (a[i] <= t)
			a[i] = w1;
		else
			a[i] = w2;
	};

	return;
}
