/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/file11.cpp $
 $LastChangedRevision: 85 $
 $Date: 2014-07-02 16:07:08 -0400 (Wed, 02 Jul 2014) $
 $Author: agulati $
 ***********************************************************
 */

#define _LARGEFILE_SOURCE
#define _FILE_OFFSET_BITS 64

#include <cstdlib>
#include <cstdio>
#include <cstring>

#include "blkdta.h"
#include "creacm.h"
#include "geom.h"
#include "spctrm.h"
#include "noise.h"
#include "objects.h"
#include "consts.h"
#include "uiod.h"

#include "int2str.h"

#include "file11.h"

File11_class File11;

void File11_class::CreaInPicture()
{

	static const INTEGER pict = CHAR2INT('p', 'i', 'c', 't');

	GetName();

	// READ ENERGY SPECTRAL DISTRIBUTION
	ReadEnergySpectralDistribution();

	// READ OBJECTS MAKING UP THE PHANTOM
	ReadObjects(pict);

	// GET INFORMATION ABOUT CREATION OF PHANTOM
	GetInformationABoutCreationOfPhantom(pict);

	return;
}

void File11_class::CreaInProj()
{

	static const INTEGER proj = CHAR2INT('p', 'r', 'o', 'j');

	GetName();

	// READ ENERGY SPECTRAL DISTRIBUTION
	ReadEnergySpectralDistribution();

	// READ OBJECTS MAKING UP THE PHANTOM
	ReadObjects(proj);

	// READ RAYSUM INFORMATION
	ReadRaysumInformation();

	if (!Creacm.prjflg)
	{
		return;
	}

	// READ IN AVERAGING INFORMATION FOR COMPUTING RAYSUMS
	ReadIfoForComputionRaysums();

	// INPUT ENTIRE GEOMETRY DATA
	ReadGeometryData(proj);

	// READ NUMBER OF RAYS AND RAY SPACING
	ReadNumberOfRaysAndRaySpacing(proj);

	// READ NUMBER OF PROJECTIONS
	// READ LIST OF PROJECTION ANGLES INTO WORK AREA
	ReadProjectionAngles(TRUE);

	// MEASUREMENT STATISTICS  --  NOISE AND SCATTER
	GetMesurementStatistics();

	// READ BACKGROUND ABSORPTION
	ReadBackgroundAbsorption();
}


bool File11_class::ReadProjection(REAL* beta, REAL* ibase, INTEGER usrays)
{
 REAL val;

 if(fscanf(infile, "%lf", &val) != 1)
    return FALSE;

 *beta = val;

 // skip angle in degrees
 if(fscanf(infile, "%lf", &val) != 1)
    return FALSE;

 for(int j = 0; j < usrays; j++){
     if(fscanf(infile, "%lf", &val) != 1){
        return FALSE;
       }
     *(ibase + j) = val;
    }

 return TRUE;
}

INTEGER File11_class::Open(BOOLEAN wr)
{
	if (wr)
	{
		infile = fopen("file11", "wb");
	}
	else
	{
		infile = fopen("file11", "rb");
	}

	if (infile == NULL)
	{
		return -1; // error opening file
	}
	return 0;
}

void File11_class::Close()
{
	fclose(infile); // !!! core dump under cygwin
}

void File11_class::Flush()
{
	fflush(infile);
}

void File11_class::Rewind()
{
	fseek(infile, 0, SEEK_SET);
}

void File11_class::WriteRowOfPhanPixels(REAL* test_aray, INTEGER nelem)
{

	for (int k = 0; k < nelem; k++)
	{
		if ((k) % 7 == 0)
		{
			fprintf(infile, "\n");
		}

		fprintf(infile, " %25.20f", test_aray[k]);

		if (test_aray[k] != 0)
		{
			fflush(infile);
		}
	}
}

void File11_class::WriteEndOfPhan()
{
	fprintf(infile, "\n");
}

void File11_class::WriteProj(REAL theta, REAL* nlo, INTEGER usrays)
{
	REAL thedeg = theta * 180 / Consts.pi;


	fprintf(infile, "\n %25.20f %25.20f", theta, thedeg);

	for (int nr = 0; nr < usrays; nr++)
	{
		if ((nr) % 3 == 0)
		{
			fprintf(infile, "\n");
		}
		fprintf(infile, " %25.20f", nlo[nr]);
	}
}

void File11_class::WriteEndOfProj()
{
	fprintf(infile, "\n");
}

void File11_class::ReadPicture(REAL* Pict, INTEGER nelem)
{
	REAL* jlo = Pict;
	signed CHAR ch;  // added by Lajos, Nov 18, 2004

//#pragma message( "to fix!" )

	for(int i = 0; i < nelem; i++){
            REAL val;
             
            for(int j = 0; j < nelem; j++){
                if(fscanf(infile, "%lf", &val) != 1){
                   fprintf(output,"\n **** end of file encountered on file file11");
                   fprintf(output, "\n **** program aborted\n");
                   exit(444);
                  }
                *jlo++ = val;
               }
           }

	while (((ch = fgetc(infile)) != '\n') && (ch != EOF))
	{  // added check for EOF. Lajos, Nov 18, 2004
		;
	}

}

void File11_class::wrhdr(INTEGER word)
{

	static const INTEGER keywd[40] =
	{
			CHAR2INT('l', 'a', 's', 't'),
			CHAR2INT('s', 'p', 'e', 'c'), CHAR2INT('m', 'o', 'n', 'o'),
			CHAR2INT('p', 'o', 'l', 'y'), CHAR2INT('o', 'b', 'j', 'e'),
			CHAR2INT('p', 'h', 'a', 'n'), CHAR2INT('a', 'v', 'e', 'r'),
			CHAR2INT('r', 'a', 'y', 's'), CHAR2INT('g', 'e', 'o', 'm'),
			CHAR2INT('p', 'a', 'r', 'a'), CHAR2INT('d', 'i', 'v', 'e'),
			CHAR2INT('s', 't', 'r', 'i'), CHAR2INT('l', 'i', 'n', 'e'),
			CHAR2INT('t', 'a', 'n', 'g'), CHAR2INT('a', 'r', 'c', ' '),
			CHAR2INT('u', 'n', 'i', 'f'), CHAR2INT('v', 'a', 'r', 'i'),
			CHAR2INT('a', 'n', 'g', 'l'), CHAR2INT('m', 'e', 'a', 's'),
			CHAR2INT('p', 'e', 'r', 'f'), CHAR2INT('n', 'o', 'i', 's'),
			CHAR2INT('q', 'u', 'a', 'n'), CHAR2INT('s', 'c', 'a', 't'),
			CHAR2INT('m', 'u', 'l', 't'), CHAR2INT('a', 'd', 'd', 'i'),
			CHAR2INT('s', 'e', 'e', 'd'), CHAR2INT('b', 'a', 'c', 'k'),
			CHAR2INT('r', 'u', 'n', ' '), CHAR2INT('m', 'o', 'd', 'i'),
			CHAR2INT('s', 'a', 'v', 'e'), CHAR2INT('p', 'i', 'x', 'e'),
			CHAR2INT('e', 'q', 'u', 'a'), CHAR2INT('u', 's', 'e', 'r'),
			CHAR2INT('a', 'n', 'd', ' '), CHAR2INT('a', 'f', 't', 'e'),
			CHAR2INT('s', 't', 'e', 'p'), CHAR2INT('s', 'p', 'a', 'c'),
			CHAR2INT('p', 'n', 'c', 'h'), CHAR2INT('l', 'i', 'n', 'o'), /// bug #135. Lajos, 08/02/2005
			CHAR2INT(' ', ' ', ' ', ' ')
	};

	static const CHAR keywdStr[40][5] =
	{ "last", "spec", "mono", "poly", "obje", "phan", "aver", "rays", "geom",
			"para", "dive", "stri", "line", "tang", "arc ", "unif", "vari",
			"angl", "meas", "perf", "nois", "quan", "scat", "mult", "addi",
			"seed", "back", "run ", "modi", "save", "pixe", "equa", "user",
			"and ", "afte", "step", "spac", "pnch", "lino", /// bug #135. Lajos, 08/02/2005
			"    " };

	static const CHAR kindStr[5][5] =
	{ "elip", "rect", "tria", "segm", "sect" };

	static const INTEGER KWI_last = 0;
	static const INTEGER KWI_spec = 1;
	static const INTEGER KWI_mono = 2;
	static const INTEGER KWI_poly = 3;
	static const INTEGER KWI_obj = 4;
	static const INTEGER KWI_phan = 5;
	static const INTEGER KWI_aver = 6;
	static const INTEGER KWI_ray = 7;
	static const INTEGER KWI_geom = 8;
	static const INTEGER KWI_angl = 17;
	static const INTEGER KWI_meas = 18;
	static const INTEGER KWI_perf = 19;
	static const INTEGER KWI_nois = 20;
	static const INTEGER KWI_quan = 21;
	static const INTEGER KWI_scat = 22;
	static const INTEGER KWI_mult = 23;
	static const INTEGER KWI_add = 24;
	static const INTEGER KWI_seed = 25;
	static const INTEGER KWI_back = 26;
	static const INTEGER KWI_pixel = 30;
	static const INTEGER KWI_lino = 38;  /// bug #135. Lajos, 08/02/2005

	// DATA SPACE FOR OBJECTS IN PHANTOM AT TOP OF BLANK COMMON

	/*
	 real object(17,1)
	 equivalence (object, work(2) )
	 */

	BOOLEAN perfct;
	BOOLEAN noisfl;
	BOOLEAN quanfl;
	///FILE* file;

// VARIABLES INDEXING THE KEYWORDS IN THE ARRAY KEYWD

	/*
	 integer  last, spec, mono, poly, obj, phan, aver, ray, geom, parl,
	 1   divr, stri, lin, itang, iarc, unif, vari, angl, meas, perf,
	 2   nois, quan, scat, mult, add, seed, back, run, modi, save,
	 3   pixel, equ, pnch

	 data     last, spec, mono, poly, obj, phan, aver, ray, geom, parl/
	 1         1   , 2   , 3   , 4   , 5  , 6   , 7   , 8  , 9   , 10  /
	 data divr, stri, lin, itang, iarc, unif, vari, angl, meas, perf/
	 1     11  , 12  , 13 , 14   , 15  , 16  , 17  , 18  , 19  , 20  /
	 data nois, quan, scat, mult, add, seed, back, run, modi, save /
	 1     21  , 22  , 23  , 24  , 25 , 26  , 27  , 28 , 29  , 30   /
	 data pixel, equ, pnch / 31, 32, 38/
	 */

	/*
	 integer den1, denav, type, cx, cy, u, v, ang, sinang, cosang, rayl
	 data den1,denav,type,cx,cy, u, v,ang,sinang,cosang,rayl
	 +    / 1,   8,    9,  10,11,12,13,14,   15,    16,   17/
	 */

	//static const REAL one = 1.0;
	int i, j;

	INTEGER itype;

	quanfl = NoisePar.quanin > 0;
	noisfl = quanfl || NoisePar.ultnfl || NoisePar.addnfl;
	perfct = !(NoisePar.sctnfl || noisfl);

	// SET THE OUTPUT FILE TO FILE11 OR PUNCH
	/*
	 file = file11;

	 if(word == keywd[KWI_pnch]) {
	 // ALSO SET VALUES PROPERLY FOR UNAVAILABLE QUANTITIES
	 ///    file = punch;
	 Creacm.nobj = 0;
	 Spctrm.nergy = 1;
	 Spctrm.energy[0] = -1;
	 }
	 */

	// WRITE HEADER INFORMATION ON FILE11 FOR PHANTOM & RAYSUMS
	// PUNCH OUT THE HEADER INFORMATION FOR THE PHANTOM AND PICTURES

	fprintf(infile, "%s", Creacm.name);

	// SPECTRUM
	if (Spctrm.nergy == 1)
	{
		fprintf(infile, "\n%s    %s %4i", keywdStr[KWI_spec],
				keywdStr[KWI_mono], Spctrm.energy[0]);
	}

	if (Spctrm.nergy > 1)
	{

		fprintf(infile, "\n%s    %4s %4i", keywdStr[KWI_spec],
				keywdStr[KWI_poly], Spctrm.nergy);
		fprintf(infile, "\n");

		for (i = 0; i < Spctrm.nergy; i++)
		{
			fprintf(infile, "%5i %5i ", Spctrm.energy[i], Creacm.percnt[i]);
		}
	}

	// OBJECTS

	fprintf(infile, "\n%s", keywdStr[KWI_obj]);

	if (Creacm.nobj != 0)
	{

		for (i = 0; i < Creacm.nobj; i++)
		{
			itype = (int) objects[i].type;

			fprintf(infile, "\n%4s  %9.4f %9.4f %9.4f %9.4f %9.4f %25.18f",
					kindStr[itype], objects[i].cx, objects[i].cy, objects[i].u,
					objects[i].v, objects[i].ang, objects[i].den1[0]);

			if (Spctrm.nergy > 1)
			{

				fprintf(infile, "\ndens ");
				for (j = 1; j < Spctrm.nergy; j++)
				{
					fprintf(infile, " %15.10f", objects[i].den1[j]);
				}
			}
		}
	}

	//jk 11/18/2007  introducing variability
	//the additional values from LAST card have to be printed in file11
	fprintf(infile, "\n%s %10.4f %ld %10.4f", keywdStr[KWI_last], mean,
			Creacm.varseed, variability);

	// SKIP PHANTOM AVERAGING AND OTHER INFO SUCH AS NELEM FOR RDPROJ

	if (word != keywd[KWI_ray])
	{
		if (word == keywd[KWI_phan])
		{

			fprintf(infile, "\n%4s    %4s %4i", keywdStr[KWI_phan],
					keywdStr[KWI_aver], Creacm.nave1);
			fprintf(infile, "\n%4s    %5i    size    %10.4f",
					keywdStr[KWI_pixel], GeoPar.nelem, GeoPar.pixsiz);
		}

		return;
	}

	// HEADER INFO FOR RAYSUMS
	// GEOMETRY

	fprintf(infile, "\n%4s    %4s %4i\n", keywdStr[KWI_ray], keywdStr[KWI_aver],
			GeoPar.nave2);
	for (i = 0; i < GeoPar.nave2; i++)
	{
		fprintf(infile, " %4i", GeoPar.naper[i]);
	}

	fprintf(infile, "\n%4s", keywdStr[KWI_geom]);

	if (GeoPar.lino)  /// bug #135. Lajos, 08/02/2005
	{
		fprintf(infile, "\n%s", keywdStr[KWI_lino]);
	}
	else
	{
		if (GeoPar.par)
		{
			fprintf(infile, "\n%4s", int2str(Creacm.word1));
			fprintf(infile, " %4s", int2str(Creacm.word2));
			fprintf(infile, " %4s", int2str(Creacm.word3));
		}

		if (GeoPar.div)
		{
			CHAR wrd2[5];

			strcpy(wrd2, int2str(Creacm.word2));
			fprintf(infile,
					"\n%4s    %4s    source at%10.4f     det dist%10.4f",
					int2str(Creacm.word1), wrd2, GeoPar.radius, GeoPar.stod);
		}

		fprintf(infile, "\n%s    user    %5i    spacing    %10.4f",
				keywdStr[KWI_ray], GeoPar.usrays, GeoPar.pinc);


		fprintf(infile, "\n%s    %5i\n", keywdStr[KWI_angl], GeoPar.prjnum);
		for (i = 0; i < GeoPar.prjnum; i++)
		{
			if (((i % 7) == 0) && (i != 0))
			{
				fprintf(infile, "\n");
			}
			fprintf(infile, " %9.4f", Creacm.pang[i]);
		}

	}

	// MEASUREMENT STATISTICS

	if (perfct)
	{
		fprintf(infile, "\n%s    %s", keywdStr[KWI_meas], keywdStr[KWI_perf]);
	}


	else
	{
		fprintf(infile, "\n%s    %s", keywdStr[KWI_meas], keywdStr[KWI_nois]);
	}

	if (quanfl)
	{
		fprintf(infile, "\n%s    %14.4f    %14.4f    cali %2i",
				keywdStr[KWI_quan], NoisePar.quanmn, NoisePar.quancm,
				NoisePar.quanin);
	}

	if (NoisePar.sctnfl)
	{
		fprintf(infile, "\n%s    noise peak %20.10f    width %20.10f",
				keywdStr[KWI_scat], NoisePar.sctnpk, NoisePar.sctnwd);
	}

	if (NoisePar.ultnfl)
	{
		fprintf(infile, "\n%s    noise mean %20.10f    std dev %20.10f",
				keywdStr[KWI_mult], NoisePar.ultnmn, NoisePar.ultnsd);
	}

	if (NoisePar.addnfl)
	{
		fprintf(infile, "\n%s    noise mean %20.10f    std dev %20.10f",
				keywdStr[KWI_add], NoisePar.addnmn, NoisePar.addnsd);
	}

	if (!perfct && noisfl)
	{
		fprintf(infile, "\n%s    %4i", keywdStr[KWI_seed], Creacm.iseed);
	}

	fprintf(infile, "\n%4s    ", keywdStr[KWI_back]);
	for (i = 0; i < Spctrm.nergy; i++)
	{
		fprintf(infile, " %7.4f", Spctrm.backgr[i]);
	}

}

