/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/foru_frfilt.cpp $
 $LastChangedRevision: 85 $
 $Date: 2014-07-02 16:07:08 -0400 (Wed, 02 Jul 2014) $
 $Author: agulati $
 ***********************************************************
 */

#include <cstdio>
#include <cmath>

#include "blkdta.h"
#include "geom.h"
#include "consts.h"
#include "sinc.h"

#include "foru.h"

void foru_class::frfilt(REAL* vector, INTEGER size1, INTEGER nfiltr, REAL cutoff, REAL phi)
{
	REAL wvqntm;
	REAL wvnum;
	INTEGER k;
	REAL fact;

	wvqntm = (REAL) 1.0 / ((REAL) (size1 / 2));
	if (GeoPar.vri)
		wvqntm /= MAX0((REAL) fabs(cos(phi)), (REAL) fabs(sin(phi)));
	wvnum = 0.;
	switch (nfiltr)
	{
	case 0:
		for (k = 0; k <= size1 - 2; k += 2)
		{  // changed "<" to "<=" to match Fortran code. Lajos, Jan 13, 2005
			if (wvnum > cutoff)
			{
				vector[k] = 0.;
				vector[k + 1] = 0.;
			}
			wvnum += wvqntm;
		}
		return;

	case 1:
		for (k = 0; k <= size1 - 2; k += 2)
		{  // changed "<" to "<=" to match Fortran code. Lajos, Jan 13, 2005
			if (wvnum > cutoff)
			{
				vector[k] = 0.;
				vector[k + 1] = 0.;
			}
			else
			{
				fact = sinc(wvnum / cutoff);
				vector[k] *= fact;
				vector[k + 1] *= fact;
			}
			wvnum += wvqntm;
		}
		return;

	case 2:
		for (k = 0; k <= size1 - 2; k += 2)
		{  // changed "<" to "<=" to match Fortran code. Lajos, Jan 13, 2005
			if (wvnum > cutoff)
			{
				vector[k] = 0.;
				vector[k + 1] = 0.;
			}
			else
			{
				fact = (REAL) cos(Consts.pid2 * wvnum / cutoff);
				vector[k] *= fact;
				vector[k + 1] *= fact;
			}
			wvnum += wvqntm;
		}
		return;

	case 3:
		for (k = 0; k <= size1 - 2; k += 2)
		{  // changed "<" to "<=" to match Fortran code. Lajos, Jan 13, 2005
			if (wvnum > 1.)
			{
				vector[k] = 0.;
				vector[k + 1] = 0.;
			}
			else
			{
				fact = cutoff
						+ ((REAL) 1.0 - cutoff) * (REAL) cos(Consts.pi * wvnum);
				vector[k] *= fact;
				vector[k + 1] *= fact;
			}
			wvnum += wvqntm;
		}
		return;
	}
}
