/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/foru_intrpl.cpp $
 $LastChangedRevision: 85 $
 $Date: 2014-07-02 16:07:08 -0400 (Wed, 02 Jul 2014) $
 $Author: agulati $
 ***********************************************************


 INTRPL: INTERPOLATION
 PURPOSE: GIVEN TWO TRANSFORMED PROJECTIONS IN ARRAY STARTING AT
 NOLD AND NEW WITH ANGLES ANG1 AND ANG2(DANG1 AND DANG2
 ARE ANGLES IN DEGREE ) FIND ALL POINTS THAT LIE BETWEEN
 THE TWO RAYS AND PERFORM INTERPOLATIONS
 */

/*      
 DETERMINE THE CASE:
 ICASE = 1 IF THE TWO ANGLES LIE BETWEEN -90 AND -45
 2 IF THE TWO ANGLES STRADDLE THE -45 DEGREE LINE
 3 IF THE TWO ANLES LIE BETWEEN -45 AND 45 LINES
 4 IF THE TWO ANGLES STRADDLE THE 45 DEGREE LINE
 5 IF THE TWO ANGLES LIE BETWEEN THE 45 AND 90 LINES
 6 IF ANG1 IS IN 1ST QUADRANT AND ANG2 IN 4TH QUADRANT
 */

#include <cstdio>

#include "blkdta.h"
#include "geom.h"
#include "uiod.h"

#include "foru.h"

void foru_class::intrpl(REAL ang1, REAL ang2, REAL dang1, REAL dang2, REAL* nold, REAL* new1)
{
	INTEGER icase;

	if (dang2 <= -45.)
		icase = 1;
	if ((dang1 < -45.) && (dang2 > -45.))
		icase = 2;
	if ((dang1 >= -45.) && (dang2 <= 45.))
		icase = 3;
	if ((dang1 < 45.) && (dang2 > 45.))
		icase = 4;
	if (dang1 >= 45.)
		icase = 5;
	if (dang1 > dang2)
		icase = 6;

	// IF UNI THEN CALL THE INTERPOLATION ROUTINE UNIF
	// IF VRI THEN CALL THE ROUTINE VCASE1,...,OR VC56 DEPENDING ON ICASE
	if (!GeoPar.vri)
	{
		unif(icase, ang1, ang2, nold, new1);

		return;
	}
	else
	{
		switch (icase)
		{

		case 1:
			vcase1(ang1, ang2, nold, new1);
			return;

		case 2:
			vcase2(ang1, ang2, nold, new1);
			return;

		case 3:
			vcase3(ang1, ang2, nold, new1);
			return;

		case 4:
			vcase4(ang1, ang2, nold, new1);
			return;

		case 5:
		case 6:
			vc56(icase, ang1, ang2, nold, new1);
			return;
		}
	}
}
