/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/foru_vint14.cpp $
 $LastChangedRevision: 85 $
 $Date: 2014-07-02 16:07:08 -0400 (Wed, 02 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 ------------------------------------------------------------------

 PURPOSE: GIVEN THE COORDINATES OF A POINT (XG,YG) AND ITS FOUR
 NEIGHBORS (XX(I),YY(I)) CALCULATE THE INTERPOLATED VALUE
 FOR INTERP=1 AND 4

 ------------------------------------------------------------------
 */

#include <cstdio>
#include <cmath>

#include "blkdta.h"
#include "fourie.h"
#include "consts.h"

#include "foru.h"

#define dis(xa,ya,xb,yb) ((REAL) sqrt(((xa)-(xb))*((xa)-(xb))+((ya)-(yb))*((ya)-(yb))))

void foru_class::vint14(
REAL* xx,
REAL* yy,
REAL xg,
REAL yg,
REAL* pj1,
REAL* pj2,
INTEGER indmin,
INTEGER indmax,
REAL* gr, // changed to pass address. Lajos, Feb 10, 2005
		REAL* gi // changed to pass address. Lajos, Feb 10, 2005
		)
{
	REAL d1 = dis(xx[1], yy[1], xg, yg);
	REAL d2 = dis(xx[2], yy[2], xg, yg);
	REAL d3 = dis(xx[3], yy[3], xg, yg);
	REAL d4 = dis(xx[4], yy[4], xg, yg);

	// changed to dereference "gr" and "gi" in order to return computed values. Lajos, Feb 10, 2005
	if (Fourie.interp != 4)
	{

		// INTERP=1

		//REAL d = amin1(d1,d2,d3,d4);
		REAL d = MIN0(MIN0(d1, d2), MIN0(d3, d4));

		if (d == d4)
			*gr = pj2[indmax];
		if (d == d4)
			*gi = pj2[indmax + 1];

		if (d == d3)
			*gr = pj2[indmin];
		if (d == d3)
			*gi = pj2[indmin + 1];

		if (d == d2)
			*gr = pj1[indmax];
		if (d == d2)
			*gi = pj1[indmax + 1];

		if (d == d1)
			*gr = pj1[indmin];
		if (d == d1)
			*gi = pj1[indmin + 1];
		return;
	}
	else
	{
		// INTERP=4

		REAL e1 = d2 * d3 * d4;
		REAL e2 = d1 * d3 * d4;
		REAL e3 = d1 * d2 * d4;
		REAL e4 = d1 * d2 * d3;

		if (e1 <= Consts.zero)
			e1 = 0.0;
		if (e2 <= Consts.zero)
			e2 = 0.0;
		if (e3 <= Consts.zero)
			e3 = 0.0;
		if (e4 <= Consts.zero)
			e4 = 0.0;

		REAL e = e1 + e2 + e3 + e4;

		*gr = (pj1[indmin] * e1 + pj1[indmax] * e2 + pj2[indmin] * e3
				+ pj2[indmax] * e4) / e;

		*gi = (pj1[indmin + 1] * e1 + pj1[indmax + 1] * e2
				+ pj2[indmin + 1] * e3 + pj2[indmax + 1] * e4) / e;

		return;
	}
}
