/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/hit_ratio.c $
 $LastChangedRevision: 80 $
 $Date: 2014-07-01 21:01:54 -0400 (Tue, 01 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 Previously part of SuperSNARK
 */

#include "malloc.h"
#include "experimenter.h"
#include "read_eval_phantom1.h"
#include "read_eval_recon1.h"
#include "hit_ratio.h"

/* ----------------------------- hit_ratio.c -------------------------------

 This function computes the hit-ratio.

 INPUTS:
 itr1 - array of length niters containing the iteration numbers for the
 first algorithm.
 itr2 - array of length niters containing the iteration numbers for the
 second algorithm.
 niters - number of iterations to be compared.
 pairs - array of length numpairs containing the indices of the structures
 which are paired.
 numpairs - number of pairs of structures.
 keywrd1 - string containing the keyword which defines the first algorithm.
 keywrd2 - string containing the keyword which defines the second algorithm.

 OUTPUTS:
 hit_ratio1 - array of length niters containing the hit ratio for
 the first algorithm.
 hit_ratio2 - array of length niters containing the hit ratio for
 the second algorithm.

 */

void hit_ratio(int* itr1, int* itr2, int niters, int* pairs, int numpairs,
		char* keywrd1, char* keywrd2, double* hit_ratio1, double* hit_ratio2)
{
	double *phantom, *recon1, *recon2;
	int *regions, numstr, i, j, hit1, hit2, *strarea;

	/* read in abnormality index for phantom structures */

	read_eval_phantom1(&regions, &numstr, &phantom, &strarea);

	/* allocate memory to store abnormality indexes for
	 structures in the reconstructions*/

	recon1 = (double *) malloc(numstr * sizeof(double));
	recon2 = (double *) malloc(numstr * sizeof(double));

	for (i = 0; i < niters; i++)
	{ /* read in abnormality indexes for structures in
	 the reconstructions. Initialize the arrays
	 which will store the structural accuracy */

		read_eval_recon1(&itr1[i], keywrd1, numstr, recon1);
		read_eval_recon1(&itr2[i], keywrd2, numstr, recon2);
		hit_ratio1[i] = 0.0;
		hit_ratio2[i] = 0.0;

		for (j = 0; j < numpairs * 2; j += 2)
		{ /* compute hit-ratio for both algorithms */

			hit1 = (phantom[pairs[j] - 1] > phantom[pairs[j + 1] - 1])
					== (recon1[pairs[j] - 1] > recon1[pairs[j + 1] - 1]);
			hit2 = (phantom[pairs[j] - 1] > phantom[pairs[j + 1] - 1])
					== (recon2[pairs[j] - 1] > recon2[pairs[j + 1] - 1]);

			hit_ratio1[i] = hit_ratio1[i] + hit1;
			hit_ratio2[i] = hit_ratio2[i] + hit2;
		}
		hit_ratio1[i] = hit_ratio1[i] / numpairs; /* hit */
		hit_ratio2[i] = hit_ratio2[i] / numpairs; /* ratio */
	}

	free(phantom); /* free memory for next function call */
	free(recon1);
	free(recon2);
	free(strarea);
}
