/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/infile.cpp $
 $LastChangedRevision: 80 $
 $Date: 2014-07-01 21:01:54 -0400 (Tue, 01 Jul 2014) $
 $Author: agulati $
 ***********************************************************
 */

#include <cstdlib>       
#include <cstdio>
#include <cctype>

#include "blkdta.h"
#include "uiod.h"
#include "creacm.h"
#include "geom.h"

#include "anglst.h"
#include "creaer.h"
#include <cmath> // to go
#include "consts.h" // to go

#include "infile.h"
#include "int2str.h"

InFile_class InFile;

BOOLEAN InFile_class::GetFormat(BOOLEAN NewLine, CHAR* pictf)
{
	if (fscanf(infile, "%s", pictf) != 1)
	{
		return FALSE;
	}

	int c;

	while (((c = fgetc(infile)) != '\n') && (c != EOF))
	{ // added check for EOF. Lajos, Nov 18, 2004
	}

	return TRUE;
}

void InFile_class::GetComment(CHAR* head)
{
	signed CHAR ch; // changed "CHAR" to "signed CHAR". Lajos, Nov 18, 2004
	int i;

	fprintf(output, "\n");

	for (i = 0; i < 80; i++)
	{
		if (((ch = fgetc(infile)) == '\n') || (ch == EOF))
		{
			break;
		}
		head[i] = (CHAR) ch;
	}

	// read the rest of the line if more than 80 characters
	while ((ch != '\n') && (ch != EOF))
	{ // added check for EOF. Lajos, Nov 18, 2004
		ch = fgetc(infile);
	}

	head[i] = 0;
}

// READ THE ITERATION FLAG CARD AND ECHO A DECODED VERSION OF IT.

void InFile_class::listit(INTEGER* iter)
{
	int n, i;
	INTEGER list[50];
	CHAR nextch;


	// get next line
	getnxt(TRUE);

	ptr++;

	for (i = 0; i < 51; i++)
	{
		nextch = data[ptr++];
		if (nextch == 0)
		{ // if end of line encountered
			break;
		}
		if (!isdigit(nextch))
		{ // if not digit
			iter[i] = 0; // set to zero
		}
		else
		{
			iter[i] = nextch - '0'; // convert to digit
		}
	}

	for (; i < 51; i++)
	{
		iter[i] = 0;
	}

	n = 0;
	for (i = 1; i < 51; i++)
	{

		if (iter[i] == 0)
			continue;
		list[n++] = i - 1;
	}

	if (n != 0)
	{
		fprintf(output, "\n         iterations ");

		for (int i = 0; i < n; i++)
		{
			if ((i != 0) && (i % 10 == 0))
			{
				fprintf(output, "\n                    ");
			}
			fprintf(output, " %3i", list[i] + 1);
		}
	}

	if (iter[0] != 0)
	{
		fprintf(output, "\n         last iteration\n");

	}
	return;

}

void InFile_class::GetRunOptions()
{
	BOOLEAN eol;
	INTEGER tword;

	static const INTEGER run_codes[3] =
	{ CHAR2INT('r', 'u', 'n', ' '), CHAR2INT('m', 'o', 'd', 'i'), CHAR2INT('s',
			'a', 'v', 'e') };

	tword = getwrd(TRUE, &eol, run_codes, 1);

	if (tword != run_codes[0])
		creaer(run_codes[0]);

	tword = getwrd(FALSE, &eol, &(run_codes[1]), 2);
	Creacm.modif = (tword == run_codes[1]);
	Creacm.crt12 = (tword == run_codes[2]);

}

void InFile_class::CreaInCrea()
{

	static const INTEGER crea = CHAR2INT('c', 'r', 'e', 'a');

	GetName();

	// READ ENERGY SPECTRAL DISTRIBUTION
	ReadEnergySpectralDistribution();

	// READ OBJECTS MAKING UP THE PHANTOM
	ReadObjects(crea);

	// GET INFORMATION ABOUT CREATION OF PHANTOM
	// THIS IS NOT RELEVANT FOR RDPROJ.  SO SKIP IN THE CASE OF RDPROJ
	GetInformationABoutCreationOfPhantom(crea);

	// READ RAYSUM INFORMATION
	ReadRaysumInformation();

	if (!Creacm.prjflg)
	{

		/// original snark does not allow combinig pseudo with RUN options
#ifdef SEMINAR_RUN_OPTIONS
		GetRunOptions(); // !!!! for seminar
#endif
		return;
	}

	// READ IN AVERAGING INFORMATION FOR COMPUTING RAYSUMS

	fprintf(output, "\n         this run will generate projection data");

	ReadIfoForComputionRaysums();

	// INPUT ENTIRE GEOMETRY DATA
	ReadGeometryData(crea);

	// READ NUMBER OF RAYS AND RAY SPACING
	ReadNumberOfRaysAndRaySpacing(crea);

	// READ NUMBER OF PROJECTIONS
	// READ LIST OF PROJECTION ANGLES INTO WORK AREA
	ReadProjectionAngles(FALSE);

	// MEASUREMENT STATISTICS  --  NOISE AND SCATTER
	GetMesurementStatistics();

	// READ BACKGROUND ABSORPTION
	ReadBackgroundAbsorption();

	// IS THIS A MODIFY RUN OR DOES IT CREATE A FILE TAPE12 FOR FUTURE
	fprintf(output, "\n ");
	GetRunOptions();
}

void InFile_class::CreaInProj()
{

	static const INTEGER proj = CHAR2INT('p', 'r', 'o', 'j');

	GetName();

	// INPUT ENTIRE GEOMETRY DATA
	ReadGeometryData(proj);

	// READ NUMBER OF RAYS AND RAY SPACING
	ReadNumberOfRaysAndRaySpacing(proj);

	// READ NUMBER OF PROJECTIONS
	// READ LIST OF PROJECTION ANGLES INTO WORK AREA
	ReadProjectionAngles(FALSE);

	// MEASUREMENT STATISTICS  --  NOISE AND SCATTER
	GetMesurementStatistics();

	// READ BACKGROUND ABSORPTION
	ReadBackgroundAbsorption();
}

