/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/quad_mtamxp.cpp $
 $LastChangedRevision: 80 $
 $Date: 2014-07-01 21:01:54 -0400 (Tue, 01 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 Y = SA*M#*A*(M*X - P)

 X AND Y MUST BE DISTINCT
 */

#include <cstdio>
#include <cmath>

#include "blkdta.h"
#include "geom.h"
#include "fourie.h"
#include "noise.h"
#include "anglst.h"
#include "wray.h"
#include "ray.h"
#include "errfac.h"
#include "consts.h"

#include "blob.h"
#include "quad.h"

void quad_class::mtamxp(REAL* x, REAL* y, INTEGER* list, REAL* weight)
{
	BOOLEAN alg;

	INTEGER i;
	INTEGER np;
	INTEGER nr;
	INTEGER numb;
	REAL snorm;
	REAL val;
	INTEGER nb;
	INTEGER k;
	REAL truval;
	REAL pix2;
	REAL pix3;
	REAL pix4;
	REAL theta;
	REAL sinth;
	REAL costh;
	REAL rinc2;
	REAL hex_voronoi;
	hex_voronoi = Consts.sqrt3 / (REAL) 2.0 * Blob.delta * Blob.delta;

	INTEGER area;

	if (Blob.pix_basis)
	{

		area = GeoPar.area;
	}
	else
	{
		area = Blob.area;
	}

	///tracer
	//for(i = 0; i < area; i++) {
	//   fprintf(output,"\nmtamxpx=%f", x[i]);
	// }
	////tracer

	// ZERO OUT Y

	for (i = 0; i < area; i++)
	{
		y[i] = 0.0;
	}

	// FOR BETTER PERFORMANCE LINE AND STRIP MODES ARE SEPERATED

	if (!GeoPar.strip)
	{

		// LINE MODE

		for (np = 0; np < GeoPar.prjnum; np++)
		{
			for (nr = 0; nr < GeoPar.nrays; nr++)
			{

				if (Blob.pix_basis)
				{
					wray(np, nr, list, weight, &numb, &snorm);
				}
				else
				{
					Blob.bwray(np, nr, list, weight, &numb, &snorm);
				}
				if (numb > 0)
				{
					val = 0.0;

					// VAL = M*X

					for (nb = 0; nb < numb; nb++)
					{
						k = list[nb];
						val += x[k] * weight[nb];
					}

					// VAL = M*X - P

					truval = Anglst.prdta(np, nr);
					val -= truval;

					// VAL = SA*A*(M*X - P)

					if ((eopt - 2) >= 0)
					{
						if ((eopt - 2) > 0)
						{
							error_1 = uerror(np, nr, truval, &alg);
						}
						else
						{
							error_1 = errfac(truval, 0.0, 0.0);
						}
					}

					val *= sa / error_1;

					// Y = SA*M#*A*(M*X - P)

					for (nb = 0; nb < numb; nb++)
					{
						k = list[nb];
						y[k] += val * weight[nb];
					}

//    NEXT NR
				}
			}

//    NEXT NP

		}

		return;
	}
//    STRIP MODE

	Fourie.rinc = GeoPar.pinc;  //think rinc and these pix* for blob case. hstau
	if (Blob.pix_basis)
	{
		pix2 = GeoPar.pixsiz * GeoPar.pixsiz;
		pix3 = pix2 * sa;
		pix4 = pix3;   // bug in Fortran?. NO hstau
	}
	else
	{
		pix2 = hex_voronoi;
		pix3 = pix2 * sa;
		pix4 = pix3;
	}

	for (np = 0; np < GeoPar.prjnum; np++)
	{
		Anglst.getang(np, &theta, &sinth, &costh);
		if ((NoisePar.quanin > 0) && GeoPar.vri)
			Fourie.rinc = GeoPar.pinc
					* MAX0((REAL) fabs(costh), (REAL) fabs(sinth));
		rinc2 = Fourie.rinc * Fourie.rinc;

		for (nr = 0; nr < GeoPar.nrays; nr++)
		{
			if (Blob.pix_basis)
			{
				ray(np, nr, list, weight, &numb, &snorm);
			}
			else
			{
				Blob.bray(np, nr, list, weight, &numb, &snorm);
			}

			if (numb > 0)
			{
				val = 0.0;

				// VAL = M*X  (NOTE THAT M IS A 0-1 MATRIX)

				for (nb = 0; nb < numb; nb++)
				{
					k = list[nb];
					val += x[k];
				}

				val *= pix2;

				// VAL = M*X - P

				truval = Anglst.prdta(np, nr);

				val -= truval;

				// VAL = SA*M#*A*(M*X - P)

				if ((eopt - 2) >= 0)
				{
					if ((eopt - 2) > 0)
					{
						error_1 = uerror(np, nr, truval, &alg);
					}
					else
					{
						error_1 = errfac(truval, Fourie.rinc, rinc2);
					}
				}
				val *= pix4 / error_1;

				// Y = SA*M#*A*(M*X - P)

				for (nb = 0; nb < numb; nb++)
				{

					k = list[nb];
					y[k] += val;
				}

				// NEXT NR
			}

		}

		// NEXT NP

	}
	return;
}
