/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/quad_nxtpct.cpp $
 $LastChangedRevision: 89 $
 $Date: 2014-07-02 17:24:53 -0400 (Wed, 02 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 COMPUTE NEXT PICTURE
 */

#include <cstdio>
#include <cmath>
#include "blkdta.h"
#include "geom.h"
#include "modefl.h"
#include "consts.h"
#include "uiod.h"
#include "blob.h"
#include "quad.h"

void quad_class::nxtpct(REAL* x, REAL* cg, REAL delta)
{

	INTEGER i;
	REAL prjden;
	REAL temden;
	REAL tem;
	REAL ratio;
	REAL snorm;
	REAL corr;

	INTEGER area;
	//INTEGER pr;

	//pr = (Blob.H + Blob.M)%2;

	if (Blob.pix_basis)
	{
		area = GeoPar.area;
	}
	else
	{
		area = Blob.area;
	}

	for (i = 0; i < area; i++)
	{
		x[i] -= delta * cg[i];
	}

	// CONSTRAIN VALUES AND AVERAGE DENSITY ADJUSTMENT

	prjden = GeoPar.aveden * ((REAL) (GeoPar.area));

	temden = 0.0;

	for (i = 0; i < area; i++)
	{
		if (Blob.pix_basis || (!Blob.pix_basis && i % 2 == Blob.pr))
		{
			tem = x[i];
			if (Modefl.lofl)
				tem = MAX0(tem, Modefl.lower);
			if (Modefl.upfl)
				tem = MIN0(tem, Modefl.upper);
			temden += tem;
			x[i] = tem;
		}
	}

	if (hopt >= 1)
	{

		if (fabs(temden) <= Consts.zero)
		{
			fprintf(output,
					"\n\n      reconstructed picture is zero, your request for picture normalizing set the picture to aveden");

			for (i = 0; i < area; i++)
			{
				x[i] = GeoPar.aveden;
			}

		}
		else
		{

			ratio = prjden / temden;
			if (popt >= 1)
			{
				fprintf(output, "\n\n      ratio of densities ,%15.4e", ratio); //bug 147, wei, 2005/10
			}

			for (i = 0; i < area; i++)
			{
				x[i] *= ratio;
			}
		}
	}

	if (popt <= 0)
		return;

//     CALCULATE INDICATOR SIZE OF CORRECTION

	snorm = 0.0;
	for (i = 0; i < area; i++)
	{
		if (Blob.pix_basis || (!Blob.pix_basis && i % 2 == Blob.pr))
		{
			snorm += cg[i] * cg[i];
		}
	}

	snorm = (REAL) sqrt(snorm);
	corr = (REAL) fabs(delta) * snorm;

	fprintf(output, "\n\n      norm of correction ,%15.4e", corr); //bug 147, wei, 2005/10
	fprintf(output, "\n      delta              %15.4e", delta);
	fprintf(output, "\n      norm of corr vector %15.4e", snorm);

	return;
}
