/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/read_eval_recon1.c $
 $LastChangedRevision: 92 $
 $Date: 2014-07-02 17:46:48 -0400 (Wed, 02 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 Previously part of SuperSNARK
 */

#include <string.h>
#include <strings.h>

#include "experimenter.h"
#include "errorc.h"
#include "read_eval_recon1.h"

/* ----------------------------- read_eval_recon1.c ----------------------

 This function reads from the eval file the abnormality index for
 all the structures in a reconstruction. The abnormality index for a
 structure is defined to be the average pixel value for those pixels
 whose centers are within that structure.

 INPUTS:
 iter - iteration number
 keywrd - keyword which defines the algorithm whose reconstruction is
 to be read in.
 numstr - number of structures in reconstruction.

 OUTPUT:
 recon - array of length numstr containing the abnormality index for each
 structure in the reconstruction.
 */

void read_eval_recon1(int* iter, char* keywrd, int numstr, double* recon)
{
	int reg, area, i, tmp_iter;
	double average;
	char string[MAXLINESIZE], iter_text[MAXLINESIZE], test[MAXLINESIZE];
	char * tmp_iter_ptr;
	FILE *evalfile;

	if (*iter == 0)
	{
		/* this means we want the LAST iteration of the given algorithm;
		 * first we need to figure out the number of this iteration though;
		 *
		 * 1) scan the eval file until the like containing the keywrd for the
		 * algorithm is found
		 * 2) keep scanning until the last iteration is found
		 */

		if ((evalfile = fopen("eval", "r")) == NULL)
		{
			errorc("in read_eval_recon1.c: error in opening file ", "eval");
		}

		while (1)
		{
			/* Read eval file until a line which contains the string
			 'execution name' is reached. If this line contains the
			 keyword break out of the while loop. If end of file is
			 reached then keyword was not found so terminate program. */

			fgets(string, sizeof(string), evalfile);
			if (strstr(string, "execution name") != NULL)
			{
				sscanf(string, "%*s%*s %s", test);
				if (strncasecmp(test, keywrd, 4) == 0)
				{
					break;
				}
			}
			if (feof(evalfile))
			{
				sprintf(string, "Error in read_eval_recon1.c: keyword %s not "
						"found in file", keywrd);
				errorc(string, " eval");
			}
		}

		while (1)
		{
			/* Continue reading eval file until a line which contains
			 the string 'Metrics for algorithm' is reached. If this
			 line contains the string 'iteration' and the required
			 iteration number break out of the while loop. If end
			 of file is reached then the appropriate string was
			 not found so print error message and exit */

			fgets(string, sizeof(string), evalfile);
			if ((strstr(string, "metrics for algorithm") != NULL) && // changed "m" in "metrics for algorithm" to lowercase. Lajos, Dec 16, 2004
					((tmp_iter_ptr = strstr(string, "iteration")) != NULL))
			{
				//grab the iteration number from this line
				tmp_iter_ptr = tmp_iter_ptr + 9;
				if (0 == sscanf(tmp_iter_ptr, "%d", &tmp_iter))
					sprintf(string,
							"Error in read_eval_recon1.c: iteration number missing");
			}

			if ((strstr(string, "execution name") != NULL) || (strstr(string, "execution name") != NULL))
			{
				//we got to the next algorithm section, so better save the
				//number of last iteration and break
				*iter = tmp_iter;
				break;
			}

		}
		fclose(evalfile); /* close eval file */
	}

	/* convert iteration number to a string */

	sprintf(iter_text, "%d", *iter);

	if ((evalfile = fopen("eval", "r")) == NULL)
	{
		errorc("in read_eval_recon1.c: error in opening file ", "eval");
	}

	while (1)
	{
		/* Read eval file until a line which contains the string
		 'execution name' is reached. If this line contains the
		 keyword break out of the while loop. If end of file is
		 reached then keyword was not found so terminate program.  */

		fgets(string, sizeof(string), evalfile);
		if (strstr(string, "execution name") != NULL)
		{
			sscanf(string, "%*s%*s %s", test);
			if (strncasecmp(test, keywrd, 4) == 0)
			{
				break;
			}
		}
		if (feof(evalfile))
		{
			sprintf(string, "Error in read_eval_recon1.c: keyword %s not found in file", keywrd);
			errorc(string, " eval");
		}
	}

	while (1)
	{
		/* Continue reading eval file until a line which contains
		 the string 'Metrics for algorithm' is reached. If this
		 line contains the string 'iteration' and the required
		 iteration number break out of the while loop. If end
		 of file is reached then the appropriate string was
		 not found so print error message and exit */

		fgets(string, sizeof(string), evalfile);
		if ((strstr(string, "metrics for algorithm") != NULL)
				&& // changed "m" in "metrics for algorithm" to lowercase. Lajos, Dec 16, 2004
				(strstr(string, iter_text) != NULL)
				&& (strstr(string, "iteration") != NULL))
		{
			break;
		}

		if (feof(evalfile))
		{
			sprintf(string, "Error in read_eval_recon1.c: %s ---> iteration %d not found in", keywrd, *iter);
			errorc(string, " eval");
		}
	}

	while (1)
	{
		/* Continue reading eval file until a line of numbers is
		 reached then break out of while loop. Terminate program
		 if end of file is reached */

		fgets(string, sizeof(string), evalfile);
		if (sscanf(string, "%d %d %lf", &reg, &area, &average) == 3)
		{
			break;
		}
	}

	if (feof(evalfile))
	{
		errorc("Error in read_eval_recon1.c: end of file reached in file ",
				"eval");
	}
	/* this is the abnormality index of the first structure
	 in the reconstruction */

	recon[0] = average;

	/* now read in the abnormality index for the remaining
	 structures */

	for (i = 1; i < numstr; i++)
	{
		fgets(string, sizeof(string), evalfile);
		sscanf(string, "%d %d %lf", &reg, &area, &average);
		recon[i] = average;
	}

	fclose(evalfile); /* close eval file */
}
