/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/recfile.cpp $
 $LastChangedRevision: 118 $
 $Date: 2014-07-09 14:22:29 -0400 (Wed, 09 Jul 2014) $
 $Author: agulati $
 ***********************************************************
 */

#include <cstring>
#include <cstdlib>       
#include <cstdio>
#include <cctype>

#include "int2str.h"
#include "DIGFile.h"
#include "modefl.h"
#include "recfile.h"

SnarkRecFile RecFile;

void SnarkRecFile::FlipImage(REAL* Dst, REAL* Src)
{
	unsigned INTEGER i;
	REAL* SrcPtr = Src;  // first row
	REAL* DstPtr = Dst + NumberOfElements * (NumberOfElements - 1); // last row

	for (i = 0; i < NumberOfElements; i++)
	{

		// copy row of data
		memcpy(DstPtr, SrcPtr, NumberOfElements * sizeof(REAL));

		DstPtr -= NumberOfElements; // set to previous row
		SrcPtr += NumberOfElements; // set to next row
	}
}

INTEGER SnarkRecFile::Open(const CHAR* FileName, INTEGER nelem, REAL pixsiz)
{
	// opens a file for writing
	// and sets main header values

	NumberOfElements = nelem;

	// allocate data buffer
	DataBuffer = new REAL[NumberOfElements * NumberOfElements];

	PixelSize = pixsiz;

	if (DIGFileSnarkRec::Open(FileName, "Proj Name", nelem, pixsiz, "This reconstruction file has been created by snark14") != 0)
	{
		return 1;
	}
	return 0;
}

INTEGER SnarkRecFile::Open(const CHAR* FileName)
{
	// opens a file for reading

	if (DIGFileSnarkRec::Open(FileName) != 0)
	{
		return 1;
	}

	if (GetSampling(&PixelSize) != 0)
	{
		return 2;
	}

	if (GetDimensions(&NumberOfElements) != 0)
	{
		return 3;
	}

	DataBuffer = new REAL[NumberOfElements * NumberOfElements];

	RecNum = 0;
	RecSetNum = 0;

	return 0;
}

INTEGER SnarkRecFile::GetNelem(INTEGER* nelem)
{

	// returns the value of nelem

	*nelem = NumberOfElements;

	return 0;
}

INTEGER SnarkRecFile::GetPixSize(REAL* pixsize)
{

	// returns the value of nelem

	*pixsize = PixelSize;
	return 0;
}

INTEGER SnarkRecFile::GetProjName(CHAR pProjName[81])
{
	// returns the projection name

	const CHAR* ProjName;

	if (DIGFileSnarkRec::GetProjName(&ProjName) != 0)
	{
		return -1;
	}

	strncpy(pProjName, ProjName, 80);

	pProjName[80] = 0;

	return 0;
}

INTEGER SnarkRecFile::SetProjName(CHAR* pProjName)
{
	// changes the projection name (already set when opening file for writing)
	return DIGFileSnarkRec::SetProjName(pProjName);
}

INTEGER SnarkRecFile::WritePhan(CHAR* pPhanName, REAL* pData)
{
	// flip image to DIGFile order
	FlipImage(DataBuffer, pData);

	// append to DIGFile
	return AppendPhantom(pPhanName, "Phantom created by snark14", DataBuffer);
}

INTEGER SnarkRecFile::WriteRec(CHAR* pRecName, CHAR pAlgName[4],
		unsigned INTEGER pCount, unsigned INTEGER pIter, REAL* pData)
{

	// writes a reconstruction array
	// opens a new array set if necessary

	// trim string for 3 letter algorithm names
	if (pAlgName[3] == ' ')
	{
		pAlgName[3] = 0;
	}

	// convert to upper case
	for (int i = 0; i < 4; i++)
	{
		pAlgName[i] = toupper(pAlgName[i]);
	}

	if (pCount == 1)
	{ // new  array set
		if (AppendRecSet(pAlgName, pRecName, "") != 0)
		{
			return 1;
		}
	}

	// flip image to DIGFile order
	FlipImage(DataBuffer, pData);

	if (AppendRec(pCount, "Reconstruction created by snark14", DataBuffer) != 0)
	{
		return 2;
	}

	return 0;
}

INTEGER SnarkRecFile::ReadPhan(CHAR pPhantomName[81], REAL* pData)
{
	// reads a phatom array (first array set)
	// if none exists, returns 2

	const CHAR* PhantomName;

	if (!PhantomPresent)
	{ // no phantom in file
		return 2;
	}

	if (GetPhantomName(&PhantomName) != 0)
	{
		return 3;
	}

	strncpy(pPhantomName, PhantomName, 80);

	pPhantomName[80] = 0;

	if (GetPhantomData(DataBuffer) != 0)
	{
		return 5;
	}

	// flip image to Snark order
	FlipImage(pData, DataBuffer);

	unsigned int ArraySize;

	GetArrayNoOfItems(&ArraySize);

	ApplyLimits(ArraySize, pData);

	return 0;
}

INTEGER SnarkRecFile::ApplyLimits(unsigned int pArraySize, REAL* pData)
{
	unsigned int i;

	if (Modefl.lofl)
	{
		if (Modefl.upfl)
		{
			for (i = 0; i < pArraySize; i++)
			{
				if (pData[i] < Modefl.lower)
					pData[i] = Modefl.lower;
				if (pData[i] > Modefl.upper)
					pData[i] = Modefl.upper;
			}
		}
		else
		{
			for (i = 0; i < pArraySize; i++)
			{
				if (pData[i] < Modefl.lower)
					pData[i] = Modefl.lower;
			}
		}
	}
	else
	{
		if (Modefl.upfl)
		{
			for (i = 0; i < pArraySize; i++)
			{
				if (pData[i] > Modefl.upper)
					pData[i] = Modefl.upper;
			}
		}
	};

	return 0;
}

INTEGER SnarkRecFile::ReadRec(CHAR pRecName[81], CHAR pAlgName[4],
		unsigned INTEGER* pCount, unsigned INTEGER* pIter, REAL* pData)
{
	// reads the next reconstruction array

	const char* AlgName;
	const char* RecName;
	unsigned INTEGER NoOfRecs;

	// Reconstruction set
	if (SelectRecSet(RecSetNum) != 0)
	{
		return 7;
	}

	// Reconstruction set header
	if (GetRecSetName(&RecName) != 0)
	{
		return 1;
	}

	strncpy(pRecName, RecName, 80);

	pRecName[80] = 0;

	// Reconstruction set header
	if (GetRecSetAlgName(&AlgName) != 0)
	{
		return 3;
	}

	pAlgName[0] = 0;

	strncat(pAlgName, AlgName, 4);

	// get number of recs in rec set
	if (GetNoOfRecs(&NoOfRecs))
	{
		return 5;
	}

	// select reconstruction within reconstruction set
	if (DIGFileSnarkRec::SelectRec(RecNum) != 0)
	{
		return 8;
	}

	/*
	 if(GetRecCount(pCount) != 0) {
	 return 4;
	 }
	 */
	*pCount = RecNum + 1;

	if (*pCount == NoOfRecs)
	{
		*pIter = 1;
	}
	else
	{
		*pIter = (*pCount - 1) % 50 + 2;
	}

	if (GetRecData(DataBuffer) != 0)
	{
		return 6;
	}

	// flip image to Snark order
	FlipImage(pData, DataBuffer);

	unsigned int ArraySize;

	GetArrayNoOfItems(&ArraySize);

	ApplyLimits(ArraySize, pData);

	if (RecNum == (NoOfRecs - 1))
	{  // if - end of recnstruction set
		RecSetNum++;
		RecNum = 0;
	}
	else
	{                          // not end of recnstruction set
		RecNum++;
	}
	return 0;
}

// closes the file	
INTEGER SnarkRecFile::Close()
{
	if (DataBuffer != NULL)
		delete[] DataBuffer;

	return DIGFile::Close();
}

INTEGER SnarkRecFile::SelectRec(unsigned INTEGER pRecNo)
{
	// selects a reconstruction
	// input: array number in file
	// output: will set the array set number and the array number
	// first array is 0

	unsigned INTEGER NoOfRecSets;
	unsigned INTEGER NoOfRecs;
	unsigned INTEGER RecSetNo;
	unsigned INTEGER RecNo;

	if (GetNoOfRecSets(&NoOfRecSets) != 0)
	{
		return -1; // error
	}

	RecNo = pRecNo;

	for (RecSetNo = 0; RecSetNo < NoOfRecSets; RecSetNo++)
	{

		// select reconstruction set
		if (SelectRecSet(RecSetNo) != 0)
		{
			return -1; // error
		}

		// get number of reconstructions in set
		if (GetNoOfRecs(&NoOfRecs) != 0)
		{
			return -1; // error
		}

		// Check if reconstruction in current set
		if (RecNo < NoOfRecs)
		{ // reconstruction in curent reconstrction set

			RecSetNum = RecSetNo;
			RecNum = pRecNo;

			// select reconstruction within reconstruction set
			if (DIGFileSnarkRec::SelectRec(RecNum) != 0)
			{
				return -1;
			}
			return 0;
		}

		RecNum -= NoOfRecs;
	}

	return -1; // requested reconstruction number is too high
}
