/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/wray.cpp $
 $LastChangedRevision: 121 $
 $Date: 2014-07-09 17:52:58 -0400 (Wed, 09 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 DEFINITION OF VARIABLES
 NUMB      THE NUMBER OF PIXELS A RAY PASSES THROUGH
 SNORM     SUM OF SQUARES OF WEIGHTS
 WEIGHT    THE LENGTH OF A RAY IN A PIXEL (CORRESPONDING TO
 ENTRY IN THE ARRAY  LIST)
 THETA     THE ANGLE IN RADIANS OF THE SOURCE OF THE RAY
 WITH RESPECT TO THE PICTURE AREA
 SORSTOC   THE DISTANCE FROM THE SOURCE TO THE CENTER OF THE BOD
 THE SELECTED PIXELS
 D         THE DISTANCE (ON THE TARGET )  FROM THE PERPENDICULAR
 THROUGH THE SOURCE TO THE PLACE AT WHICH THE
 DENSITY VALUE WAS MEASURED
 LIST      THE LIST OF INDECIES INTO THE ARRAY PICT THAT CORRESP
 TO THE SILECTED PIXELS

 05/21/2008      jk      added 4 default parameters
 this allows to pass sorx, sory, cf and sf and avoid
 a call to posit() - used when taking projections
 of phantoms with variability when the call to posit()
 is made right before the call wray()
 */

#include <cstdio>
#include <cmath>

#include "blkdta.h"
#include "geom.h"
#include "uiod.h"

#include "posit.h"

#include "wray.h"

void wray(INTEGER np, INTEGER nr, INTEGER* list, REAL* weight, INTEGER* numb, REAL* snorm, REAL * _sorx, REAL * _sory, REAL * _cf, REAL * _sf)
//_sorx, _sory, _cf and _sf are default parametes with values set to REAL * _sorx=0, REAL * _sory=0, REAL * _cf=0, REAL * _sf=0
{
	INTEGER flip;

	REAL scale;
	REAL elem;
	REAL sorx;
	REAL sory;
	REAL cf;
	REAL sf;
	REAL hx;
	REAL hy;
	REAL tanphi;
	REAL rtnphi;
	REAL hxhx;
	REAL hyhy;
	REAL c;
	INTEGER ix;
	INTEGER iy;
	REAL fp;
	INTEGER incx;
	REAL w;
	REAL yc;
	REAL xc;
	INTEGER index;

	int next;

	scale = GeoPar.pixsiz;
	elem = (REAL) GeoPar.nelem;
	*numb = 0;
	*snorm = 0;

	//jk  5/21/2008
	//if the values for _sorx and _sory are provided, do not call posit()
	if (_sorx != 0 && _sory != 0 && _cf != 0 && _sf != 0)
	{
		sorx = *_sorx;
		sory = *_sory;
		cf = *_cf;
		sf = *_sf;
	}
	else
		posit(np, nr, &sorx, &sory, &cf, &sf);

	sorx /= GeoPar.pixsiz;
	sory /= GeoPar.pixsiz;

	if (sf < 0.0)
	{
		sf = -sf;
		cf = -cf;
	}

	flip = 0;
	if (sf * cf < 0.0)
	{
		flip = 1;
		cf = -cf;
	}

	/*
	 IF SIN(PHI) IS TOO CLOSE TO ZERO SET TO SOME SMALL VALUE
	 WITH PROPER SIGN
	 LIKEWISE WITH COS(PHI)  THIS AVOIDS TROUBLES WITH DIVISION
	 AND INFINITE SLOPES
	 */

	if (fabs(sf) <= 0.0000001)
	{
		if (sf < 0)
		{
			sf = (REAL) -0.0000001;
		}
		else
		{
			sf = (REAL) 0.0000001;
		}
	}

	hx = ((REAL) 1.0) / sf;

	if (((REAL) fabs(cf)) <= (REAL) 0.0000001)
	{
		if (cf < 0)
		{
			cf = (REAL) -0.0000001;
		}
		else
		{
			cf = (REAL) 0.0000001;
		}
	}


	hy = ((REAL) 1.0) / cf;
	tanphi = sf / cf;
	rtnphi = cf / sf;
	hx *= scale;
	hxhx = hx * hx;
	hy *= scale;
	hyhy = hy * hy;

	// CALCULATE THE INTERSECTION OF REFERANCE LINE AND THE RAY IN QUESTI

	sory += elem / ((REAL) 2.0);

	if (flip > 0)
	{
		// REFLECT IN Y=0 IF NEEDED
		sorx = -sorx;
	}

	sorx += elem / ((REAL) 2.0);
	c = sory - sorx * tanphi;

	// FLIP LINE 180 DEGREES

	c = (((REAL) 1.0) - tanphi) * elem - c;

	// EQUATION OF RAY IS NOW Y=X*TANPHI+C
	// C IS THE Y INTERCEPT
	// IF INTERCEPT IS ABOVE (N) THEN THE RAY HAS MISSED ALL PIXELS

	if ((c - elem) < 0)
	{
		if (c < 0)
		{

			// IF Y INTERCEPT IS NEGATIVE THEN TAKE X INTERCEPT (WHICH MUST
			// BE POSITIVE SINCE SLOPE IS ALWAYS POSITIVE

			//  RAY ENTERS FROM BOTTOM

			c = -c * rtnphi;

			// RECALCULATE C SO THAT EQUATION OF LINE IS NOW
			// X=RTANPHI*Y+C
			// IF X INTERCEPT IS GREATER TNAN N THE RAY HAS MISSED THE RECONSTRUC
			// AREA

			if ((c - elem) >= 0)
				goto L360;

			// INITIAL GRID (IX,IY) COORDINATE IS NOW CALCULATED

			ix = (INTEGER) c;
			iy = 0;

			// FP IS ACTUAL DISPLACEMENT OF INTERCEPT FROM THE NEXT GREATER
			// GRID LINE

			fp = ((REAL) (ix + 1)) - c;

			// START AT LOW X VALUES AND INCREMENT ACROSS PICTURE UNLESS
			// SLOPE IS NEGATIVE

			incx = 1;

			// FLIP INDICATOR IS EXACTLY WRONG FOR HERMANS MATRIX STORAGE

			if (flip <= 0)
			{

				// START AT HIGH X VALUES AND DECREMENT (FROM RIGHT TO LEFT)

				ix = GeoPar.nelem - ix - 1;
				incx = -1;
			}

			// SET INITIAL VALUES OF THE LENGTH OF THE RAY IN CURRENT CELL

			w = hy * fp;
			yc = fp * tanphi;
			xc = -fp;

			// INDEX IS THE LINEAR ARRAY SUBSCRIPT OF THE  PICTURE ARRAY
			// INDEX INTO FORTRAN TYPE ARRAY

			index = iy * GeoPar.nelem + ix + 1;

			// INITIAL CONDITIONS SET FOR ENTRANCE TO LOOPS WE HAVE ENTERED
			// THE PICTURE AREA FROM THE BOTTOM

			next = 3;
		}
		else
		{

			// ENTER PICTURE FROM SIDE (LEFT)
			// CALCULATE INITIAL GRID COORDINATES (IX,IY)

			ix = 0;
			incx = 1;

			// START WITH LOW X VALUES AND GO UP UNLESS THE FLIP INDICATOR
			// IS SET
			// FLIP INDICATOR IS EXACTLY WRONG FOR HERMANS MATRIX STORAGE

			if (flip <= 0)
			{
				// START WITH HIGH X VALUES AND DECREMENT

				ix = GeoPar.nelem - 1;
				incx = -1;
			}

			iy = (INTEGER) c;

			// FP IS DISTANCE FROM Y INTERCEPT TO NEXT GREATER GRID LINE

			fp = ((REAL) (iy + 1)) - c;

			// CALCULATE ARRAY SUBSCRIPT IN PICTURE AREA OF INITIAL CELL
			// INDEX INTO FORTRAN TYPE ARRAY

			index = iy * GeoPar.nelem + ix + 1;

			// CALCULATE INITIAL DESCRIPTION OF RAY IN FIRST CELL FOR LOOP

			yc = -fp;
			w = hx * fp;
			xc = fp * rtnphi;

			// RAY ENTERS FROM LEFT SIDE

			next = 1;
		}

		for (;;)
		{
			switch (next)
			{
			case 1:
				//  MAKE ROOM FOR NEW PICTURE ELEMENT
				(*numb)++;
				list[(*numb) - 1] = index - 1;

				// STEP ONE IN X DIRECTION AND CORRESPONDING STEP IN Y DIRECTION

				xc -= (REAL) 1.0;
				yc += tanphi;

				// IF XC IS NEGATIVE THE Y GRID LINE HAS BEEN CROSSED THE RAY
				// HAS LEFT THROUGHTHE TOP

				if (xc > (REAL) 0.0)
				{
					// ENTERED SIDE,LEFT THROUGH SIDE --ASSIGN MAXIMUM Y-TYPE WEIGHT
					weight[(*numb) - 1] = hy;
					(*snorm) += hyhy;
					w -= hy;
					next = 4;
				}
				else
				{
					weight[(*numb) - 1] = w;
					(*snorm) += w * w;

					if ((w - (REAL) 0.0000001) < 0)
					{
						w = hy;
						(*numb)--;
					}
					else
					{
						w = hy - w;
					}

					next = 2;
				}
				break;
				//////////////////////////////////////////////////////////////
			case 2:
				iy++;

				// RAY ENTERS FROM BOTTOM
				// STEP ONE CELL IN Y DIRECTION  AND TEST FOR COMPLETION

				if ((iy - GeoPar.nelem) >= 0)
					goto L360;

				index += GeoPar.nelem;

				next = 3;
				break;

				//////////////////////////////////////////////////////////////

				// NEW LIST ELEMENT MEANS ANOTHER PIXEL SELECTED
			case 3:
				(*numb)++;
				list[(*numb) - 1] = index - 1;

				// STEP ONE IN Y DIRECTION AND CORRESPONDING X INCREMENT

				yc -= 1.0;
				xc += rtnphi;

				// IF YC IS NEGATIVE WH HAVE CROSSED A X-GRID LINE AND THAT
				// WE LEAVE THROUGE THE SIDE

				if (yc > (REAL) 0.0)
				{
					w -= hx;

					// ENTERED BOTTOM LEFT TOP ASSIGN MAXIMUM X-TYPE WEIGHT

					weight[(*numb) - 1] = hx;
					(*snorm) += hxhx;
					next = 2;
				}
				else
				{
					weight[(*numb) - 1] = w;
					(*snorm) += w * w;

					// IF THE WEIGHT SO CALCULATED IS TOO SMALL DELETE THIS PIXEL
					// THIS CAUSED BY CROSSING GRID LINES AT A CORNER

					if ((w - (REAL) 0.000001) <= 0.0)
					{
						w = hx;
						(*numb)--;
					}
					else
					{
						w = hx - w;
					}
					next = 4;
				}
				break;

				/////////////////////////////////////////////////////////////
			case 4:
				// ENTER PIXEL FROM SIDE
				ix += incx;

				// STEP ONE IN X DIRECTION
				// AND TEST FOR TERMINATION

				if (ix < 0)
					goto L360;

				if ((ix - GeoPar.nelem) >= 0)
					goto L360;
				index += incx;

				next = 1;
				break;
			}
		}
	}

	L360:

	if (trace > 6)
	{
		fprintf(output, "\n          wray    np = %5i  nr = %5i  numb = %5i  snorm = %9.4f", np, nr, *numb, *snorm);
	}
	if ((*numb > 0) && (trace > 9))
	{
		for (int i = 0; i < *numb; i++)
		{
			fprintf(output, "\n                  %5i     %9.4f", list[i], weight[i]);
		}
	}
}
