/** @file chooseYVar.hpp
    @package snark14Display
    @author Bruno M. Carvalho and Deniz Sarioz
    licensed under (open-source) QPL v1.0
    which accompanies this distribution in the file QPL
*/

#ifndef CHOOSEYVAR_H
#define CHOOSEYVAR_H

#include "variables.hpp"
#include "displaywidget.hpp"

#include <qdialog.h>

#include <qbuttongroup.h>
#include <qcolor.h>
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qfiledialog.h>
#include <qfont.h>
#include <qframe.h>
#include <qimage.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qlineedit.h>
#include <qlistbox.h>
#include <qmessagebox.h>
#include <qpainter.h>
#include <qpen.h>
#include <qpixmap.h>
#include <qpoint.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qtooltip.h>
#include <qvariant.h>
#include <qwhatsthis.h>
#include <qwidget.h>

class QVBoxLayout;
class QHBoxLayout;
class QButtonGroup;
class QCheckBox;
class QComboBox;
class QFrame;
class QGridLayout; 
class QLabel;
class QLineEdit;
class QListBox;
class QListBoxItem;
class QPushButton;
class QRadioButton;
class QPixmap;

class chooseYVarWindow : public QDialog
{
  Q_OBJECT

public:
  chooseYVarWindow( QWidget* parent = 0, const char* name = 0, bool modal = true, WFlags fl = 0 ,int x = 0 , int y = 0 );
  ~chooseYVarWindow();
  int id;
  bool status;
  globalYVar_t globY;
  unsigned PBPY;
  //  double maxval,minval,diffmaxval,diffminval;
  displayWidget* displaywidget;
  QBoxLayout* topLayout;
  QBoxLayout* yaxisbuttonLayout;
  QBoxLayout* x_extrema_layout;
  QBoxLayout* y_extrema_layout;
  QButtonGroup* yaxisbuttongroup;
  QCheckBox* sameExtremaForAllCheckBox;
  QRadioButton* globalbutton;
  QComboBox* globalerrors;
  QRadioButton* pointbutton;
  QComboBox* pointerrors;
  QLabel* minxlabel;
  QLabel* maxxlabel;
  QLabel* minylabel;
  QLabel* maxylabel;
  QLineEdit* minx;
  QLineEdit* maxx;
  QLineEdit* miny;
  QLineEdit* maxy;
  QPushButton* proceedbutton;
  QPushButton* closebutton;
  QButtonGroup* colorbuttongroup;
  QRadioButton* grayScaleButton;
  QRadioButton* colorButton;
private:
  void spawnPlots(bool plotPBP, int minXVal, int maxXVal, double minYVal, double maxYVal);
  void spawnDialogs(bool plotPBP);
  void activateExtremaFields();
  void deactivateExtremaFields();
  void getCalculatedExtrema();
public slots:
  void activateGlobalErrors();
  void activatePointErrors();
  void maybeGetCalculatedExtrema();
  void sameExtremaClicked();
  void proceed();
  void closeWindow();
    // jk 2/2/2009 adding grayScale option for graphs
  bool isGrayScaleChecked();
protected:
  virtual void closeEvent(QCloseEvent *e);
};
#endif // CHOOSEYVAR_H
