/** @file displaywindow.cpp
    @package snark14Display
    @author Bruno M. Carvalho and Deniz Sarioz
    licensed under (open-source) QPL v1.0
    which accompanies this distribution in the file QPL
        
    modification: Joanna Klukowska 1/13/2009
	    added choice of grayScale display for plot
	    in addition to color display
*/

#include "variables.hpp"
#include "verbosity.hpp"
#include "displaywidget.hpp"
#include "displaywindow.hpp"
#include "lines.hpp"
#include "line_real_t.hpp"
#include "plot_t.hpp"
#include "displaylines.hpp"

#include <iostream>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <qcolor.h>
#include <qfiledialog.h>
#include <qfont.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qlineedit.h>
#include <qmessagebox.h>
#include <qpainter.h>
#include <qpalette.h>
#include <qpen.h>
#include <qpixmap.h>
#include <qpoint.h>
#include <qpushbutton.h>
#include <qslider.h>
#include <qstyle.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qwidget.h>
#include <qregexp.h> 
#include <qscrollview.h>

#define EPSILON 0.00000001

#ifndef MIN
#define MIN(a, b) (((a < b) ? (a) : (b)))
#endif

// extern displaywindow **displaywindowset;
// QImage rec; // this needs to be an INSTANCE VARIABLE
// bool converted; // wouldn't hurt to make this an INSTANCE VARIABLE either (but it's unused??)

/* 
 *  Constructs a displaywindow which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
displaywindow::displaywindow( QWidget* parent, const char* name, bool modal, WFlags fl , DIGDataFormat dataformat, int imagenumber, int x, int y )
  //    : QDialog( parent, name, modal, fl )
  : QWidget(parent, name, fl)
{
		      // -Qt::WStyle_Customize | Qt::WStyle_NoBorder
//    std::cerr << "++displaywindow constructor for imagenumber " << imagenumber << std::endl;
    int i,j;
    QPixmap disp;
    recp = 0; // initialize
    // double **phantomimage;
    double **phantomimage;
    QPoint topleft;
    id=imagenumber;
    //    displaywindowset[id]=this; // don't need this.
    setName(imagetitles[id]);
    imagename=QString(imagetitles[id]);
    setCaption(imagetitles[id]);
    //    displaywindowactive[id]=true;
    status=false;
    topleft.setX(x);
    topleft.setY(y);
    this->move(topleft);
    imageset[imagenumber]->readImage(imagenumber,dataformat);
    //printf("max=%f min=%f \n",imageset[imagenumber]->getMax(),imageset[imagenumber]->getMin());
    if(imageset[imagenumber]->getMax() > globalmax) {
      globalmax=imageset[imagenumber]->getMax();
    }
    if(imageset[imagenumber]->getMin() < globalmin) {
      globalmin=imageset[imagenumber]->getMin();
    }
    maxzoom = 2 * maxSizex / Sizex;
    if(maxzoom<=1) maxzoom = 2;
    image = imageset[imagenumber]->getImage();
    lowthreshold=minval=imageset[imagenumber]->getMin();
    highthreshold=maxval=imageset[imagenumber]->getMax();
    range=highthreshold-lowthreshold;
    Range=highthreshold-lowthreshold;
    if(phantomexists && id) { // (id) means "image number is not 0" (phantom must be 0)
      diffimage=(double **)malloc(Sizey*sizeof(double *));
      if (!diffimage) {
	std::cerr << "Failed to allocate space for diffimage" << std::endl;
      }
      for(j=0;j<Sizey;j++) {
        diffimage[j]=(double *)malloc(Sizex*sizeof(double));
        if(!diffimage[j]) {
	  std::cerr << "Failed to allocate space for diffimage" << std::endl;
        }
      }  
      if(!imageset[0]->isLoaded()) {
	imageset[0]->readImage(0,dataformat);
	if(imageset[0]->getMax()>globalmax)
	  globalmax=imageset[0]->getMax();
	if(imageset[0]->getMin()<globalmin)
	  globalmin=imageset[0]->getMin();
      }
      phantomimage=imageset[0]->getImage();
      diffminval=0;
      diffmaxval=0;
      for(i=0;i<Sizey;i++) {
        for(j=0;j<Sizex;j++) {
          diffimage[i][j]=image[i][j]-phantomimage[i][j];
          if(diffimage[i][j]>diffmaxval)
	    diffmaxval=diffimage[i][j];
          if(diffimage[i][j]<diffminval)
	    diffminval=diffimage[i][j];
        }
      }
      if(diffmaxval > globaldiffmax) globaldiffmax = diffmaxval;
      if(diffminval < globaldiffmin) globaldiffmin = diffminval;
    }

    int fudge = 105; // "dynamic" replacement of stuff
    
    setMinimumSize(200, 500); // good enough
    //    setMinimumSize( QSize( 190 + MIN(Sizex, maxSizex) +10, 
    //			   20 + MIN(Sizey,maxSizey) ) );
    //    resize( QSize( 190+MIN(Sizex, maxSizex), 
    //		   20 + MIN(Sizey,maxSizey) ) ); // no point in this
    // no maximum size!
    if(recp) {
      std::cerr << "Error: recp already assigned!" << std::endl;
    }
    recp = new QImage(Sizex,Sizey,8,256);
    //    rec = QImage(Sizex,Sizey,8,256);
    QImage& rec = (*recp);
    for(i=0;i<256;i++)
      rec.setColor(i,qRgb(i,i,i));
    for(i=0;i<Sizey;i++)
      for(j=0;j<Sizex;j++)
        *(rec.scanLine(i)+j)=(unsigned char)(((image[i][j]-lowthreshold)/range)*255);

    disp = QPixmap(Sizex,Sizey);
    converted=disp.convertFromImage(rec,QPixmap::Auto);

    //    displaywidget->setGeometry( QRect( 190, 10, Sizex, Sizey ) ); 
    displaywidget = new displayWidget( this, "displaywidget" );
    displaywidget->setBackgroundPixmap( disp );
    //displaywidget->setScaledContents( TRUE );
    QObject::connect(displaywidget,SIGNAL(clicked(int,int)),this,SLOT(updateClickedPixel(int,int)));
    zoomlabel = new QLabel( this, "zoomlabel" );
    zoomlabel->setGeometry( QRect( 65, 5, 45, 20 ) ); 
    zoomlabel->setText( tr( "Zoom" ) );

    zoomslider = new QSlider( this, "zoomslider" );
    zoomslider->setGeometry( QRect( 5, 30, 175, 20 ) ); 
    zoomslider->setOrientation( QSlider::Horizontal );
    zoomslider->setRange(1,maxzoom);
    zoomslider->setValue(1);
    zoomslider->setLineStep(1);
    zoomslider->setPageStep(2);
    QObject::connect(zoomslider,SIGNAL(valueChanged(int)),this,SLOT(updateZoom()));
    zoomval=1;

    zoom = new QLineEdit( this, "zoom" );
    zoom->setGeometry( QRect( 5, 55, 90, 30 ) ); 
    zoom->setText("1");

    setzoombutton = new QPushButton( this, "setzoombutton" );
    setzoombutton->setGeometry( QRect( 95, 50, 90, 40 ) ); 
    setzoombutton->setText( "Set Value" );
    QObject::connect(setzoombutton,SIGNAL(clicked()),this,SLOT(updateZoomslider()));

    lowthreshlabel = new QLabel( this, "lowthreshlabel" );
    //    lowthreshlabel->setGeometry( QRect( 35, 205, 110, 20 ) ); 
    lowthreshlabel->setGeometry( QRect( 35, 205 - fudge, 110, 20 ) ); 
    lowthreshlabel->setText( tr( "Low Threshold" ) );

    lowthreshslider = new QSlider( this, "Slider4_2" );
    lowthreshslider->setGeometry( QRect( 5, 230 - fudge, 175, 20 ) ); 
    lowthreshslider->setOrientation( QSlider::Horizontal );
    lowthreshslider->setRange(0,1000000000);
    lowthreshslider->setValue(0);
    QObject::connect(lowthreshslider,SIGNAL(valueChanged(int)),this,SLOT(updateLowthresh()));

    lowthresh = new QLineEdit( this, "lowthresh" );
    lowthresh->setGeometry( QRect( 5, 260 - fudge, 90, 30 ) ); 
    char s[12];     
    sprintf(s,"%10.7f",minval);
    lowthresh->setText(s);

    setlowthreshbutton = new QPushButton( this, "setlowthreshbutton" );
    setlowthreshbutton->setGeometry( QRect( 95, 255 - fudge, 90, 40 ) ); 
    setlowthreshbutton->setText( tr( "Set Value" ) );
    QObject::connect(setlowthreshbutton,SIGNAL(clicked()),this,SLOT(updateLowthreshslider()));

    highthreshlabel = new QLabel( this, "highthreshlabel" );
    highthreshlabel->setGeometry( QRect( 35, 300 - fudge, 115, 20 ) ); 
    highthreshlabel->setText( tr( "High Threshold" ) );

    highthreshslider = new QSlider( this, "Slider4_3" );
    highthreshslider->setGeometry( QRect( 5, 325 -fudge, 175, 20 ) ); 
    highthreshslider->setOrientation( QSlider::Horizontal );
    highthreshslider->setRange(0,1000000000);
    highthreshslider->setValue(1000000000);
    QObject::connect(highthreshslider,SIGNAL(valueChanged(int)),this,SLOT(updateHighthresh()));

    highthresh = new QLineEdit( this, "highthresh" );
    highthresh->setGeometry( QRect( 5, 355 - fudge, 90, 30 ) ); 
    sprintf(s,"%10.7f",maxval);
    highthresh->setText(s);

    sethighthreshbutton = new QPushButton( this, "sethighthreshbutton" );
    sethighthreshbutton->setGeometry( QRect( 95, 350 - fudge , 90, 40 ) ); 
    sethighthreshbutton->setText( tr( "Set Value" ) );
    QObject::connect(sethighthreshbutton,SIGNAL(clicked()),this,SLOT(updateHighthreshslider()));

    pixelposlabel = new QLabel( this, "pixelposlabel" );
    pixelposlabel->setGeometry( QRect( 30, 395 - fudge, 45, 20 ) ); 
    pixelposlabel->setText( tr( "Pixel" ) );

    pixelpos = new QLineEdit( this, "pixelpos" );
    pixelpos->setGeometry( QRect( 5, 420 - fudge, 80, 30 ) ); 

    pixelvaluelabel = new QLabel( this, "pixelvaluelabel" );
    pixelvaluelabel->setGeometry( QRect( 110, 395 - fudge, 45, 20 ) ); 
    pixelvaluelabel->setText( tr( "Value" ) );

    pixelvalue = new QLineEdit( this, "pixelvalue" );
    pixelvalue->setGeometry( QRect( 90, 420 - fudge, 90, 30 ) );

    savebutton = new QPushButton( this, "savebutton" );
    savebutton->setGeometry( QRect( 15, 460 - fudge, 150, 40 ) ); 
    savebutton->setText( tr( "Save Image" ) );
    QObject::connect(savebutton,SIGNAL(clicked()),this,SLOT(saveImage()));

    linesbutton = new QPushButton( this, "linesbutton" );
    linesbutton->setGeometry( QRect( 15, 505 - fudge, 150, 40 ) ); 
    linesbutton->setText( tr( "Display Lines" ) );
    QObject::connect(linesbutton,SIGNAL(clicked()),this,SLOT(callLines()));

    closebutton = new QPushButton( this, "closebutton" );
    closebutton->setGeometry( QRect( 15, 550 - fudge, 150, 40 ) ); 
    closebutton->setText( tr( "Close Window" ) );
    QObject::connect(closebutton,SIGNAL(clicked()),this,SLOT(closeWindow()));

    scrollv = new QScrollView(this, "qscrollview", Qt::WStyle_Customize | Qt::WStyle_NoBorder );
    scrollv->viewport()->setBackgroundColor(colorGroup().background());
    scrollv->setFrameShape(QFrame::NoFrame);
    scrollv->setGeometry( 190, 10, maxSizex - 200, maxSizey - 100 );
    scrollv->addChild(displaywidget);
    useglobal = usedifference = useintensity = false;
    //    resetImage();
    // hide(); // don't need, not shown unless show() called
}  // --displaywindow ctor



/*  
 *  Destroys the object and frees any allocated resources
 */
displaywindow::~displaywindow()
{
    // no need to delete child widgets, Qt does it all for us
  displaywindowactive[id]=false;
  //  delete displaywindowset[id]; // see if this will work?
  //  displaywindowset[id]= 0; // NULL
}  // --displaywindow::~displaywindow()



/** non-destructive!  only hide the window, don't lose any information. */
void displaywindow::closeWindow()
{
  displaywindowactive[id]=false;
  hide();
}  // --displaywindow::closeWindow()



/** do the same thing as closing the window, don't self-destruct! */
void displaywindow::closeEvent(QCloseEvent * e)
{
  closeWindow();
}  // --displaywindow::closeEvent()


void displaywindow::updateClickedPixel(int cx,int cy)
{
  if(usedifference && !id) return;
  int ix,iy;
  char s[12];

  ix=cx/zoomval;
  iy=cy/zoomval;
  //printf("pixel(%d,%d) image[%d][%d]=%f\n",cx,cy,iy,ix,image[iy][ix]);
  sprintf(s,"(%d,%d)",ix,iy);
  pixelpos->setText(s);
  if(usedifference)
    sprintf(s,"%10.7f",diffimage[iy][ix]);
  else
    sprintf(s,"%10.7f",image[iy][ix]);
  pixelvalue->setText(s);
}  // --displaywindow::updateClickedPixel()



void displaywindow::updateZoom()
{
  char s[12];
  int v=zoomslider->value();
  sprintf(s,"%d",v);
  zoom->setText(s);
  zoomval=v;
  if(usedifference) {
    if(id) resetDiffImage();
    else {
      resize(MIN(maxSizex, 200+(Sizex*zoomval)),
	 MIN(maxSizey, 20+(Sizey*zoomval)));
      scrollv->resizeContents ( Sizex*zoomval + 10, Sizey*zoomval + 10 );      
    }
  } else {
    resetImage();
  }
}  // --displaywindow::updateZoom()



void displaywindow::updateZoomslider()
{
  int v;
  bool ok=false;
  v=zoom->text().toInt(&ok,10);
  if(ok && v>0 && v<=maxzoom) 
    zoomslider->setValue(v);
  else 
    QMessageBox::information(this,"SnarkDisplay","Error!\n"
                                  "Zoom value invalid or out of range.");
  updateZoom();
}  // --displaywindow::updateZoomslider()


void displaywindow::setGlobal(bool on) {
  useglobal = on;
  //  globalthresholdbutton->setChecked( on );
  lowthreshlabel->setEnabled(!on);
  lowthreshslider->setEnabled(!on);
  lowthresh->setEnabled(!on);
  setlowthreshbutton->setEnabled(!on);
  highthreshlabel->setEnabled(!on);
  highthreshslider->setEnabled(!on);
  highthresh->setEnabled(!on);
  sethighthreshbutton->setEnabled(!on);
  setDifference(usedifference); // hack!!! // we no like mutual recursion without base case
}  // --displaywindow::setGlobal()



void displaywindow::setDifference(bool on) {
  usedifference = on;
  //  differencebutton->setChecked( on );
  if(usedifference && !id) return;
  int l;
  char s[12];
  l=imagename.length();
  if(usedifference) {
    imagename.replace(l-3,1,"d");
    setCaption(imagename);
    if(!useglobal) {
      //      lowthreshslider->setMinValue((int)(diffminval*1000000000));
      //      lowthreshslider->setValue((int)(diffminval*1000000000));
      lowthreshslider->setValue(0);
      highthreshslider->setValue(1000000000);
      lowthreshold=diffminval;
      sprintf(s,"%10.7f",lowthreshold);
      lowthresh->setText(s);
      //      highthreshslider->setMaxValue((int)(diffmaxval*1000000000));
      //      highthreshslider->setValue((int)(diffmaxval*1000000000));
      highthreshold=diffmaxval;
      sprintf(s,"%10.7f",highthreshold);
      highthresh->setText(s);
      range=diffmaxval-diffminval;
      Range=diffmaxval-diffminval;
    }
    resetDiffImage();
  }
  else {
    imagename.replace(l-3,1,"r");
    setCaption(imagename);
    if(!useglobal) {
      //      lowthreshslider->setMinValue((int)(minval*1000000000));
      //      lowthreshslider->setValue((int)(minval*1000000000));
      lowthreshslider->setValue(0);
      highthreshslider->setValue(1000000000);
      lowthreshold=minval;
      sprintf(s,"%10.7f",lowthreshold);
      lowthresh->setText(s);
      //      highthreshslider->setMaxValue((int)(maxval*1000000000));
      //      highthreshslider->setValue((int)(maxval*1000000000));
      highthreshold=maxval;
      sprintf(s,"%10.7f",highthreshold);
      highthresh->setText(s);
      range=maxval-minval;
      Range=maxval-minval;
    }
    resetImage();
  }
}  // --displaywindow::setDifference()



void displaywindow::setIntensity(bool on) {
  useintensity = on;
  //  intensitybutton->setChecked(on);
  int l;
  l=imagename.length();
  if(useintensity) {
    if(id) 
      imagename.replace(l-1,1,"i");
    else
      imagename.append("i");
    setCaption(imagename);
  }
  else {
    if(id) 
      imagename.replace(l-1,1,"a");
    else
      imagename.remove(l-1,1);
    setCaption(imagename);
  }
  if(usedifference) {
    if(id) resetDiffImage();
  } else {
    resetImage();
  }
}  // --displaywindows::setIntensity()



bool displaywindow::isGlobalthreshs()
{
  return useglobal;
}  // --displaywindow::isGlobalthreshs()



void displaywindow::updateGlobalthreshs()
{
  if(useglobal) {
    if(usedifference) {
      if(id) {
	resetDiffImage();
      } else {
	resetImage();
      }
    }
  }
}  // --displaywindow::updateGlobalthreshs()



void displaywindow::setGlobalthreshs()
{
  if(usedifference && !id) return;
  if(usedifference) 
    resetDiffImage();
  else
    resetImage();
}  // --displaywindow::setGlobalthreshs()


/** update lowthreshold (number) and lowthresh (text) based on lowthreshslider */
void displaywindow::updateLowthresh()
{
  if(id || !usedifference) {
    char s[12];
    int v = lowthreshslider->value();
    if(v > highthreshslider->value()) { // cap it
      lowthreshslider->setValue(highthreshslider->value());
      lowthreshold = highthreshold;
      lowthresh->setText(highthresh->text());
    } else {
      lowthreshold = (((double)v) / 1000000000) * Range + (usedifference ? diffminval : minval);
      sprintf(s,"%10.7f",lowthreshold);
      lowthresh->setText(s);
    }
    if(usedifference) {
      resetDiffImage();
    } else {
      resetImage();
    }
  }
}  // --displaywindow::updateLowthresh()



double displaywindow::getLowthresh()
{
  return lowthreshold;
}  // --displaywindow::getLowthresh()


/** update lowthreshold and low threshold slider pos based on entered lowthresh */
void displaywindow::updateLowthreshslider()
{
  if(id || !usedifference) {
    double fv;
    int v=0;
    bool ok=false;
    fv=lowthresh->text().toDouble(&ok);
    if(ok) {
      if(fv > highthreshold) {
	QMessageBox::information(this,"SnarkDisplay","Error!\n"
				 "Cannot set low threshold higher than high threshold!");
      } else {
	if(fv < (usedifference ? diffminval : minval)) {
	  QMessageBox::information(this,"SnarkDisplay","Error!\n"
				   "Cannot set low threshold that low!");
	} else { // GOOD
	  if(fv>0) {
	    v= (int)(((fv - (usedifference ? diffminval : minval)) / Range) * 1000000000 + 0.5);
	  } else {
	    v= (int)(((fv - (usedifference ? diffminval : minval)) / Range) * 1000000000 - 0.5);
	  }
	  lowthreshslider->setValue(v);
	  lowthreshold=fv;
	  if(usedifference) {
	    resetDiffImage();
	  } else {
	    resetImage();
	  }
	  return; // if good
	}
      }
    } else { // not OK
      QMessageBox::information(this,"SnarkDisplay","Error!\n"
			       "Low Threshold value invalid.");
    }
    char s[12]; // if got here, no good one way or another.
    sprintf(s,"%10.7f",lowthreshold);
    lowthresh->setText(s); // old val
    lowthresh->setFocus();
  }
}  // --displaywindow::updateLowthreshslider()


/** update highthreshold (number) and highthresh (text) based on highthreshslider */
void displaywindow::updateHighthresh()
{
  if(id || !usedifference) {
    char s[12];
    int v = highthreshslider->value();
    if(v < lowthreshslider->value()) {
      highthreshslider->setValue(lowthreshslider->value());
      highthreshold = lowthreshold;
      highthresh->setText(lowthresh->text());
    } else {
      highthreshold = (((double)v) / 1000000000) * Range + (usedifference ? diffminval : minval);
      sprintf(s,"%10.7f",highthreshold);
      highthresh->setText(s);
    }
    if(usedifference) {
      resetDiffImage();
    } else {
      resetImage();
    }
  }
}  // --displaywindow::updateHighthresh()



double displaywindow::getHighthresh()
{
  return highthreshold;
}



void displaywindow::updateHighthreshslider()
{
  if(id || !usedifference) {
    double fv;
    int v=0;
    bool ok=false;
    fv=highthresh->text().toDouble(&ok);
    if(ok) {
      if(fv < lowthreshold) {
	QMessageBox::information(this,"SnarkDisplay","Error!\n"
				 "Cannot set high threshold lower than low threshold!");
      } else {
	if(fv > (usedifference ? diffmaxval : maxval)) {
	  QMessageBox::information(this,"SnarkDisplay","Error!\n"
				 "Cannot set high threshold that high!");
	} else { // GOOD
	  if(fv>0) {
	    v=(int)(((fv-(usedifference ? diffminval : minval))/Range)*1000000000+0.5);
	  } else {
	    v=(int)(((fv-(usedifference ? diffminval : minval))/Range)*1000000000+0.5);
	  }
	  highthreshslider->setValue(v);
	  highthreshold=fv;
	  if(usedifference) {
	    resetDiffImage();
	  } else {
	    resetImage();
	  }
	  return; // if good
	}
      }
    } else { // not OK
      QMessageBox::information(this,"SnarkDisplay","Error!\n"
			       "High Threshold value invalid.");
    }
    char s[12]; // if got here, no good one way or another.
    sprintf(s,"%10.7f",highthreshold);
    highthresh->setText(s); // old val
    highthresh->setFocus();
  }
}  // --displaywindow::updateHighthreshslider()



void displaywindow::resetImage()
{
  if(id && usedifference) return;
  double old_lowthreshold = lowthreshold;  // hack
  double old_highthreshold = highthreshold;
  if(useglobal) { // see if we can get away so easily
    lowthreshold = lowglobalthresh;
    highthreshold = highglobalthresh;
  }
  /* adjust image to current thresholds and zoom */
  int i,j,m,n,base,point;
  double t;
  unsigned char tuc;
  QPixmap disp;
  QImage& rec = (*recp);
  rec = rec.smoothScale(Sizex*zoomval,Sizey*zoomval);
  rec.setNumColors(256);
  for(i=0;i<256;i++)
    rec.setColor(i,qRgb(i,i,i));
  range=highthreshold-lowthreshold;
  /* printf("low=%f high=%f range=%f \n",lowthreshold,highthreshold,range); */

  for(i=0;i<Sizey;i++) {
    base=i*zoomval;
    for(j=0;j<Sizex;j++) {
      point=j*zoomval;
      t=((image[i][j]-lowthreshold)/range);
      //printf("t=%f\n",t);
      if(t==0.0 || t<0.0) {
        //printf("t=%f\n",t);
	for(m=0;m<zoomval;m++) 
	  for(n=0;n<zoomval;n++) 
	    *(rec.scanLine(base+m)+point+n)=0;
      }
      else {
        //printf("t=%f\n",t);
	if(t>1.0 || t==1.0) {
	  for(m=0;m<zoomval;m++) 
	    for(n=0;n<zoomval;n++) 
	      *(rec.scanLine(base+m)+point+n)=255;
	  //printf("%d\n",rec.pixelIndex(i,j));
	}
	else {
          //printf("t=%f\n",t);
	  if(useintensity) {
	    tuc=(unsigned char)(sqrt(t)*255);
	    for(m=0;m<zoomval;m++) 
	      for(n=0;n<zoomval;n++) {
	        //printf("tuc=%d\n",tuc);
	        *(rec.scanLine(base+m)+point+n)=tuc;
	        //printf("%d\n",*(rec.scanLine(i)+j));
	      }
	  }
	  else {
	    tuc=(unsigned char)(t*255);
	    for(m=0;m<zoomval;m++) 
	      for(n=0;n<zoomval;n++) {
	        //printf("tuc=%d\n",tuc);
	        *(rec.scanLine(base+m)+point+n)=tuc;
	        //printf("%d\n",*(rec.scanLine(i)+j));
	      }
	  }
	}
      }
    }
  }
  converted=disp.convertFromImage(rec,QPixmap::Auto); 
  displaywidget->resize(Sizex*zoomval,Sizey*zoomval);
  displaywidget->setBackgroundPixmap( disp );
  if(useglobal) {
    lowthreshold = old_lowthreshold;
    highthreshold = old_highthreshold;
  }
  resize(MIN(maxSizex, 200+(Sizex*zoomval)),
	 MIN(maxSizey, 20+(Sizey*zoomval)));
  scrollv->resizeContents ( Sizex*zoomval + 10, Sizey*zoomval + 10 );
} // --displaywindow::resetImage()



void displaywindow::resetDiffImage() 
{
  double old_lowthreshold = lowthreshold;  // hack
  double old_highthreshold = highthreshold;
  if(useglobal) { // see if we can get away so easily
    lowthreshold = lowglobalthresh;
    highthreshold = highglobalthresh;
  }
  /* adjust image to current thresholds and zoom */
  int i,j,m,n,base,point;
  double t;
  unsigned char tuc;
  QPixmap disp;
  QImage& rec = (*recp);
  rec = rec.smoothScale(Sizex*zoomval,Sizey*zoomval);
  rec.setNumColors(256);
  for(i=0;i<256;i++)
    rec.setColor(i,qRgb(i,i,i));
  range=highthreshold-lowthreshold;
  /* printf("low=%f high=%f range=%f \n",lowthreshold,highthreshold,range); */

  for(i=0;i<Sizey;i++) {
    base=i*zoomval;
    for(j=0;j<Sizex;j++) {
      point=j*zoomval;
      t=((diffimage[i][j]-lowthreshold)/range);
      //printf("t=%f\n",t);
      if(t==0.0 || t<0.0) {
        //printf("t=%f\n",t);
	for(m=0;m<zoomval;m++) 
	  for(n=0;n<zoomval;n++) 
	    *(rec.scanLine(base+m)+point+n)=0;
      }
      else {
        //printf("t=%f\n",t);
	if(t>1.0 || t==1.0) {
	  for(m=0;m<zoomval;m++) 
	    for(n=0;n<zoomval;n++) 
	      *(rec.scanLine(base+m)+point+n)=255;
	  //printf("%d\n",rec.pixelIndex(i,j));
	}
	else {
          //printf("t=%f\n",t);
	  if(useintensity) {
	    tuc=(unsigned char)(sqrt(t)*255);
	    for(m=0;m<zoomval;m++) 
	      for(n=0;n<zoomval;n++) {
	        //printf("tuc=%d\n",tuc);
	        *(rec.scanLine(base+m)+point+n)=tuc;
	        //printf("%d\n",*(rec.scanLine(i)+j));
	      }
	  }
	  else {
	    tuc=(unsigned char)(t*255);
	    for(m=0;m<zoomval;m++) 
	      for(n=0;n<zoomval;n++) {
	        //printf("tuc=%d\n",tuc);
	        *(rec.scanLine(base+m)+point+n)=tuc;
	        //printf("%d\n",*(rec.scanLine(i)+j));
	      }
	  }
	}
      }
    }
  }
  converted=disp.convertFromImage(rec,QPixmap::Auto); 
  displaywidget->resize(Sizex*zoomval,Sizey*zoomval);
  displaywidget->setBackgroundPixmap( disp );
  if(useglobal) {
    lowthreshold = old_lowthreshold;
    highthreshold = old_highthreshold;
  }
  resize(MIN(maxSizex, 200+(Sizex*zoomval)),
	 MIN(maxSizey, 20+(Sizey*zoomval)));
  scrollv->resizeContents ( Sizex*zoomval + 10, Sizey*zoomval + 10 );
}  // --displaywindow::resetDiffImage()



void displaywindow::resetImageRange() 
{
  /* adjust image to current thresholds */
  int i,j;
  double t;
  QPixmap disp;
  QImage& rec = (*recp);
  range=highthreshold-lowthreshold;
  /* printf("low=%f high=%f range=%f \n",lowthreshold,highthreshold,range); */

  for(i=0;i<Sizey;i++) 
    for(j=0;j<Sizex;j++) {
      t=((image[i][j]-lowthreshold)/range);
      if(t<=0)
	*(rec.scanLine(i)+j)=0;
      else 
	if(t>=1) 
	  *(rec.scanLine(i)+j)=255;
	else 
	  *(rec.scanLine(i)+j)=(unsigned char)(t*255);
    }
  converted=disp.convertFromImage(rec,QPixmap::Auto); 
  displaywidget->setBackgroundPixmap( disp );

}  // --displaywindow::resetImageRange() 



/** Starts a lines window 
@param void
@author Bruno M. Carvalho for version 1.0, Deniz
@version 1.4 
@description a lot like SnarkDisplay::callLines() defined in SnarkDisplay.cpp
*/
void displaywindow::callLines()
{
  int l[4];
  int linetype,c,i,j,minx,maxx, topleftx,toplefty;
  double **img;
  double miny,maxy;
  QString title,xaxis,yaxis,temp;
  QPoint topleft;
  imagename = imagename.simplifyWhiteSpace();
  lineswindow* nw = new lineswindow(this,imagename,lowthreshold,highthreshold,true,1);
  c=nw->exec();
  if(c==QDialog::Accepted) {
    linetype=nw->getLinetype(); // default == 0 == row; 1 == column
    // '0' means 'show for suchandsuch rows all columns within x-selection' etc
    l[0]=nw->getLine1();
    l[1]=nw->getLine2();
    l[2]=nw->getLine3();
    l[3]=nw->getLine4();
    title=nw->getTitle();    // bug 227, wei, 6/13/2007
    xaxis=nw->getXAxis();
    yaxis=nw->getYAxis();
    minx=nw->getMinX();
    maxx=nw->getMaxX();
    miny=nw->getMinY();
    maxy=nw->getMaxY();
    topleft=this->pos();
    topleftx=topleft.x();
    toplefty=topleft.y();
    QString rowOrColumn = ( (linetype) ? ("Column ") : ("Row ") );
    QString vs = ( (linetype) ? ("Rows") : ("Columns"));
    // loop through selection
    for(j=0;j<4;j++) {
      if(l[j]>=0 && l[j]<Sizex) {
	// spawn different plot for each valid j!
	line_real_set_t myLRS(minx,maxx,miny,maxy); // create a brand new LRS for brand new plot
	myLRS.clear(); // and be sure of it!
	topleftx+=30;
	toplefty+=30;
	QPixmap* pixmap = new QPixmap(sd_line_t::WIDTH, sd_line_t::HEIGHT);
	pixmap->fill(Qt::white);
        QPainter picasso;
        picasso.begin(pixmap);
        picasso.setBackgroundColor(Qt::white);
	QString whichRowOrCol;
	whichRowOrCol.setNum(l[j]);
	QString plotSuffix = (rowOrColumn + whichRowOrCol);
	//QString plotName = imagename.simplifyWhiteSpace();
	QString plotName = title.simplifyWhiteSpace();   // bug 227, wei, 6/13/2007
	plotName += (" :: " + plotSuffix);
	line_real_t ourImageLine(imagename);
	// references for unification -- have to be initialized right here
	int&row = ( (linetype) ? (i) : (l[j]) );
	int&col = ( (linetype) ? (l[j]) : (i) );

        // now do the stuff for phantom
	if((nw->isPhantomChecked()) && id) {
	  if(!imageset[0]->isLoaded())
	    imageset[0]->readImage(0,DIGDataFormat_ASCII);
	  img=imageset[0]->getImage();
	  QString phantomName = imagetitles[0];
	  phantomName = phantomName.simplifyWhiteSpace();
	  line_real_t phantomLine(phantomName);
	  for(i=0;i<Sizey;i++) {
	    phantomLine.add(i, img[row][col]);
	  }
	  myLRS.add(phantomLine);
	}


        double **&ourim = ( (usedifference) ? (diffimage) : (image) );
	for(i=0;i<Sizey;i++) {
	  ourImageLine.add(i,ourim[row][col]);
	}
	myLRS.add(ourImageLine);

	plot_t myPlot(myLRS);
	myPlot.setName(plotName);    // actual param has to be be a legitimate QString object
	myPlot.setXAxisName(xaxis);  // for all three of these names
	myPlot.setYAxisName(yaxis);
        // jk 1/13/2009 adding grayScale option for graphs
        if (nw->isGrayScaleChecked()) {
            myPlot.plotGray(picasso);
        }
        else{
            myPlot.plot(picasso);
        }
        /* Call display lines with pixmap as one of the arguments */
        displaylineswindow* dw=new displaylineswindow(this,plotName,pixmap,false,0,topleftx,toplefty);
        dw->show();
      }
    }
  }
}  // --displaywindow::callLines()



/** Saves the current image
@param void
@author Bruno M. Carvalho
@version 1.0 */ 
void displaywindow::saveImage()
{
  int i,j;
  QString prefixForSaving = imagename;
  prefixForSaving = prefixForSaving.simplifyWhiteSpace();
  prefixForSaving.replace( QRegExp("\\="), "_EQ_");
  prefixForSaving.replace( QRegExp("\\+"), "_PLUS_");
  prefixForSaving.replace( QRegExp("\\@"), "_AT_");
  //  prefixForSaving.replace( QRegExp("-"), "_MINUS_"); // maybe
  // convert everything non-alphanumeric to underscore
  prefixForSaving.replace( QRegExp("[^a-zA-Z0-9]"), "_");
  // allow maximum of 2 consecutive underscores  
  prefixForSaving.replace( QRegExp("__+") , "__"); 
  // chuck underscore from beginning and end
  prefixForSaving.replace( QRegExp("^_+"), "");
  prefixForSaving.replace( QRegExp("_+$"), "");

  QString myExtensionFilter = 
    "png (PNG file);;"
    "bmp (BMP file);;"
    "flt (ASCII float file);;"
    "pbm (PBM file);;"
    "pgm (PGM file);;"
    "ppm (PPM file);;"
    "xbm (XBM file);;"
    "xpm (XPM file)"   ;

  QString mySaveAs = "Save Image as";
  QString startInDir = "./";
  QFileDialog fd(startInDir, myExtensionFilter, this, mySaveAs, true);
  fd.setMode(QFileDialog::AnyFile);
  fd.setSelection(prefixForSaving);
  if (QDialog::Accepted != fd.exec()) {
    if(std::verbose >=2) std::cout << "--displaywindow::saveImage() in a hurry" << std::endl;
    return; // happens, say, when cancel is pressed.  just leave, no harm.
  }
  QString userFileSavePrefix = fd.selectedFile();
  QString userFileSaveExtension = fd.selectedFilter();
  if (userFileSavePrefix.isEmpty()) { // double-check for absurdity
    if(std::verbose >=2) std::cout << "--displaywindow::saveImage() in a hurry" << std::endl;
    return; // happens, say, when cancel is pressed.  just leave, no harm.
  }
  if(std::verbose >=2) std::cout << "Got file name as: \"" << userFileSavePrefix << "\"" << std::endl;
  if(std::verbose >=2) std::cout << "Got selected ext: \"" << userFileSaveExtension << "\"" << std::endl;
  /* NB: userFileSavePrefix now contains directory info as well.
   * if user entered additional forward-slashes without
   * knowing that it signified directory, that's NOT our problem.
   * (the user MAY legitimately enter "a/myImage" as file name if there is a directory "a" 
   *   within the currently open directory in the save dialog window)
   * do part only after last forward-slash (i.e., what user entered)...
   * don't want to burn ourselves if path in fact had legit non-alphanumericounderscore chars
   */
  QString justFilePrefix, justPath;
  // funnylooking because written to work with Qt BOTH 2.3 and 3.3
  int posLastSlash = userFileSavePrefix.findRev('/'); // returns pos if found, -1 if not.
  if (-1 != posLastSlash) {
    justPath = userFileSavePrefix.left(posLastSlash+1); // +1 to include last '/'
    justFilePrefix = userFileSavePrefix;
    QRegExp fakeRegex("^" + justPath); // it just *looks* fake...
    justFilePrefix.replace( fakeRegex, "" ); // chuck away including last '/' for justFilePrefix
    if(std::verbose >=2) std::cout << "just after regex replace, justFilePrefix: \"" << justFilePrefix << "\"" << std::endl;
  } else {  // if for some reason no forwardslash, handle that gracefully too.
    justFilePrefix = userFileSavePrefix;
    justPath = "";
  }
  if(std::verbose >=2) std::cout << "justPath is: \"" << justPath << "\"" << std::endl;
  if(std::verbose >=2) std::cout << "justFilePrefix is first: \"" << justFilePrefix << "\"" << std::endl;
  // remove justFilePrefix from ONLY the end of 'userFileSavePrefix'.
  justFilePrefix.replace( QRegExp("="), "_EQ_");
  justFilePrefix.replace( QRegExp("+"), "_PLUS_");
  justFilePrefix.replace( QRegExp("[^a-zA-Z0-9]"), "_"); // no forward slash in regex
  // allow maximum of 2 consecutive underscores  
  justFilePrefix.replace( QRegExp("__+") , "__"); 
  // chuck underscore from beginning and end
  justFilePrefix.replace( QRegExp("^_+"), "");
  justFilePrefix.replace( QRegExp("_+$"), "");
  if(std::verbose >=2) std::cout << "justFilePrefix is later: \"" << justFilePrefix << "\"" << std::endl;
  // reset userFileSavePrefix by putting back together justPath and the refined justFilePrefix
  userFileSavePrefix = justPath + justFilePrefix;
  userFileSaveExtension.truncate(3);
  if(std::verbose >=2) std::cout << "My userFileSavePrefix is \"" << userFileSavePrefix << "\"" << std::endl;
  if(std::verbose >=2) std::cout << "My userFileSaveExtension is \"" << userFileSaveExtension << "\"" << std::endl;
  QString extensionUpper = userFileSaveExtension.upper();
  QString saveFullNameWithPath = userFileSavePrefix + "." + extensionUpper;
  if(std::verbose >=2) std::cout << "extensionUpper is now \"" << extensionUpper << "\"" << std::endl;
  if(std::verbose >=2) std::cout << "saveFullNameWithPath is now \"" << saveFullNameWithPath << "\"" << std::endl;
  if(extensionUpper.compare(QString("FLT"))) { // if NOT "FLT" extension ("FLT" is Bruno's / Linkoping's bunchafloats format)
    QImageIO recio;
    QImage& rec = (*recp);
    recio.setImage(rec);
    recio.setFormat(extensionUpper);
    recio.setFileName(saveFullNameWithPath);
    if ( recio.write() ) {
      if(std::verbose >= 1) {
	std::cout << "displaywindow::saveImage() successfully saved: \"" << saveFullNameWithPath << "\"" << std::endl;
	return;
      }
    } else {
      std::cerr << "Error opening " << saveFullNameWithPath << std::endl;
      return;
    }
  } else {
    FILE* fp;
    if((fp = fopen(saveFullNameWithPath.latin1(),"w"))==NULL) {
      std::cerr << "UNsuccessfully tried to save as: \"" << saveFullNameWithPath << "\"" << std::endl;
      return;
    } else {
      for(i=0;i<Sizey;i++) {
	for(j=0;j<Sizex;j++) {
	  fprintf(fp,"%f ",image[i][j]);
	  if(!((j+1)%5)) 
	    fprintf(fp,"\n");
	}
	fprintf(fp,"\n");
      }
      fclose(fp);
      if(std::verbose>=1) {
	std::cout << "(we think) successfully saved: \"" << saveFullNameWithPath << "\"" << std::endl;
      }
    }
  }
}  // --displaywindow::saveImage()
